/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.model.content;

import sa.shpi.advanced.model.EmissionsItem;
import sa.shpi.advanced.model.Library;
import sa.shpi.advanced.model.Strings;

/**
 * Dynamic furniture library with presets to add to rooms. Also contains the
 * options to add "Default Room Loading", which is used in the content emissions
 * calculations.
 *
 * @author Kyle
 *
 */

public class FurnishingLibrary extends Library {

    public FurnishingLibrary() {
        initialize();
    }

    @Override
    public void initialize() {

        /**
         * Emissions items are written in the format: ("name of item", emissions
         * factor, mass)
         */

        lib.add(new EmissionsItem("*" + Strings.DEFAULT_ROOM.toString() + "*", 1.0, 1.0));
        lib.add(new EmissionsItem("Computadora", 1.8, 10.0));
        lib.add(new EmissionsItem("Portátil", 0.419, 3.5));
        lib.add(new EmissionsItem("Monitor de Computadora - CRT", 1.8, 14.1));
        lib.add(new EmissionsItem("Monitor de Computadora - LCD", 1.8, 4.7));
        lib.add(new EmissionsItem("Teléfono", 1.8, 1.0));
        lib.add(new EmissionsItem("Impresora", 1.8, 11.7));
        lib.add(new EmissionsItem("Escritorio", 1.33, 26.0));
        lib.add(new EmissionsItem("Silla", 1.9, 12.0));
        lib.add(new EmissionsItem("Mesa", 1.33, 29.0));
        lib.add(new EmissionsItem("Gabinete de Presentación - Metal", 0.756, 20.0));
        lib.add(new EmissionsItem("Gabinete de Presentación - Madera", 1.5, 20.0));
        lib.add(new EmissionsItem("Gabinete de Baño - Madera", 1.5, 7.0));
        lib.add(new EmissionsItem("Gabinete de Baño - Metal", 0.0117, 7.0));
        lib.add(new EmissionsItem("Lavabo - Cerámico", 0.0, 20.0));
        lib.add(new EmissionsItem("Lavabo - Metal", 0.0117, 20.0));
        lib.add(new EmissionsItem("Inodoro", 0.0, 30.0));
        lib.add(new EmissionsItem("Microondas", 2.5, 19));
        lib.add(new EmissionsItem("Gabinete de Cocina", 1.5, 13.5));
        lib.add(new EmissionsItem("Refrigeradora - Pequeña", 2.22, 30.0));
    }

    @Override
    public String getUnits() {
        return "kg";
    }
}
