/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.model.content;

import javax.swing.JOptionPane;
import sa.shpi.advanced.model.Strings;


/**
 * A basic room in a warehouse. Stores information relating to the
 * area of the room, the percent of damage in the room, and the furnishings in
 * the room.
 *
 * @author Kyle
 *
 */

public class Room {

    private RoomType roomType;
    private FurnishingList furnishingList;
    private int damage;
    private double roomarea;
    private int roomQuantity;
    
    public void setRoomQuantity(int roomquantity) {
        this.roomQuantity = roomquantity;
    }
    
    public int getRoomQuantity() {
        return roomQuantity;
    }

    public Room() {
        this.furnishingList = new FurnishingList();
    }

    public Room(RoomType roomType) {
        this.roomType = roomType;
        this.furnishingList = new FurnishingList();
    }

    public void setRoomType(RoomType roomType) {
        this.roomType = roomType;
    }

    public RoomType getRoomType() {
        return roomType;
    }

    public void setRoomArea(double roomarea) {
        this.roomarea = roomarea;
    }

    public double getRoomArea() {
        return roomarea;
    }

    public void addFurnishing(UsedFurnishing furnishing) {
        if (furnishing.toString().contains(Strings.DEFAULT_ROOM.toString())) {
            if (furnishingList.size() < 1) {
                furnishingList.add(furnishing);
            } else {
                JOptionPane.showMessageDialog(null, Strings.DEFAULT_PLUS);
            }
        } else if (furnishingList.toString().contains(Strings.DEFAULT_ROOM.toString())) {
            JOptionPane.showMessageDialog(null, Strings.DEFAULT_PLUS);
        } else {
            furnishingList.add(furnishing);
        }
    }

    public void setFurnishingList(FurnishingList furnishingList) {
        this.furnishingList = furnishingList;
    }

    public FurnishingList getFurnishingList() {
        return furnishingList;
    }

    public double calculateEmissions() {
        double emissions = 0.0;

        /**
         * The following calculations differentiate emissions based on type of
         * warehouse, type of affected room, and contents added to the
         * room.
         *
         * Rooms that use a default loading multiply the rooms
         * area by an average loading for the type of room.
         * Shelves that use a default loading multiply the shelfs volume by an
         * average loading for the type of product being stored (warehouse type).
         * 
         * The calculation for default room loading is written as:
         * (Average emissions (kg CO2) per unit area) * area
         *
         * Rooms that do not use a default fire loading simply total the
         * emissions from all furnishings added to the room.
         */
        
        //Differentiate emissions based on type of room.
        if (roomType.equals(RoomType.BREAK_AREA)) {
                if (furnishingList.toString().contains(Strings.DEFAULT_ROOM.toString())) {
                    emissions = 35.034 * getRoomArea();
                } else if (!furnishingList.isEmpty()) {
                    emissions = furnishingList.stream().map((furnishingList1) -> furnishingList1.calculateEmissions()).reduce(emissions, (accumulator, _item) -> accumulator + _item);
                }
        } else if (roomType.equals(RoomType.MAIN_OFFICE)) {
                if (furnishingList.toString().contains(Strings.DEFAULT_ROOM.toString())) {
                    emissions = 21.699 * getRoomArea();
                } else if (!furnishingList.isEmpty()) {
                    emissions = furnishingList.stream().map((furnishingList1) -> furnishingList1.calculateEmissions()).reduce(emissions, (accumulator, _item) -> accumulator + _item);
                }
        } else if (roomType.equals(RoomType.BATHROOM)) {
                if (furnishingList.toString().contains(Strings.DEFAULT_ROOM.toString())) {
                    emissions = 2.504 * getRoomArea();
                } else if (!furnishingList.isEmpty()) {
                    emissions = furnishingList.stream().map((furnishingList1) -> furnishingList1.calculateEmissions()).reduce(emissions, (accumulator, _item) -> accumulator + _item);
                }
        } else if (roomType.equals(RoomType.CONFERENCE)) {
                if (furnishingList.toString().contains(Strings.DEFAULT_ROOM.toString())) {
                    emissions = 20.395 * getRoomArea();
                } else if (!furnishingList.isEmpty()) {
                    emissions = furnishingList.stream().map((furnishingList1) -> furnishingList1.calculateEmissions()).reduce(emissions, (accumulator, _item) -> accumulator + _item);
                }
        } else if (roomType.equals(RoomType.CUBICLE)) {
                if (furnishingList.toString().contains(Strings.DEFAULT_ROOM.toString())) {
                    emissions = 44.849 * getRoomArea();
                } else if (!furnishingList.isEmpty()) {
                    emissions = furnishingList.stream().map((furnishingList1) -> furnishingList1.calculateEmissions()).reduce(emissions, (accumulator, _item) -> accumulator + _item);
                }
        }

        //Multiply the result by the percentage of damage.
        double roomtotal = emissions * getDamageFraction() * getRoomQuantity();
        return roomtotal;

    }

    public int getDamage() {
        return damage;
    }

    public void setDamage(int damage) {
        this.damage = damage;
    }

    public double getDamageFraction() {
        return damage * 0.01;
    }

    @Override
    public String toString() {
        return roomType.toString() + " (" + getDamage() + "% "
                + Strings.DAMAGE.toString() + ")" + " (" + String.format("%.2f", getRoomArea()) + " m^2)" + " (" + getRoomQuantity() + "x)";
    }

}
