/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.model.structure;

import sa.shpi.advanced.model.EmissionsItem;
import sa.shpi.advanced.model.Library;
import sa.shpi.advanced.model.Warehouse;

/**
 * Dynamic material library with presets to add to structural components. Also
 * contains components that can be used as default structure items.
 *
 * @author Kyle
 *
 */
public class MaterialsLibraryRoofing extends Library {

    public MaterialsLibraryRoofing() {
        initialize();
    }
    
    public void initialize() {

        /**
         * Emissions items are written in the format:
         * ("item name", emissions factor, mass per unit area)
         */
            lib.add(new EmissionsItem("Concreto", 0.00, 0.00));
            lib.add(new EmissionsItem("Acero", 0.0117, (((.00159 * Warehouse.getAreaTotal())
                    * 7850) / Warehouse.getAreaTotal())));
            lib.add(new EmissionsItem("Poliestireno - R10", 1.64, (((.0508 * Warehouse.getAreaTotal())
                    * 16) / Warehouse.getAreaTotal())));
            lib.add(new EmissionsItem("Poliestireno - R3", 1.64, (((.0508 * Warehouse.getAreaTotal())
                    * 32) / Warehouse.getAreaTotal())));
            lib.add(new EmissionsItem("Poliestireno - R1", 1.64, (((.0508 * Warehouse.getAreaTotal())
                    * 48) / Warehouse.getAreaTotal())));
    }

    @Override
    public String getUnits() {
        return "kg/m^2";
    }

}
