/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.model.structure;

/**
 * A basic room in a warehouse. Stores information relating to the
 * area of the room, the percent of damage in the room, and the furnishings in
 * the room.
 *
 * @author Kyle
 *
 */

public class Section {

    private SectionCategory secCategory;
    private MaterialList materialList;

    public Section() {
        this.materialList = new MaterialList();
    }

    public Section(SectionCategory secCategory) {
        this.secCategory = secCategory;
        this.materialList = new MaterialList();
    }

    public void setSectionCategory(SectionCategory seccat) {
        this.secCategory = seccat;
    }

    public SectionCategory getSectionCategory() {
        return secCategory;
    }

    public void addMaterial(StructureUsedMaterials furnishing) {
        materialList.add(furnishing);
    }

    public void setMaterialList(MaterialList matList) {
        this.materialList = matList;
    }

    public MaterialList getMaterialList() {
        return materialList;
    }

    public double calculateEmissions(double areaBurned) {
        double emissions = 0.0;

        if (!materialList.isEmpty()) {
            //for each material
            for (int count = 0; count < materialList.size(); count++) {
                //add the emissions result from the material
                emissions += materialList.get(count).calculateEmissions(
                        areaBurned);
            }
        }
        
        return emissions;
    }

    @Override
    public String toString() {
        return secCategory.toString();
    }

}
