/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.visual;

import javax.swing.JFrame;
import javax.swing.JTree;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JOptionPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.JButton;

import sa.shpi.SHPIImagePanel;
import sa.shpi.advanced.AdvancedTool;
import sa.shpi.advanced.model.Warehouse;
import sa.shpi.advanced.model.Strings;
import sa.shpi.advanced.model.content.FurnishingList;
import sa.shpi.advanced.model.content.Room;
import sa.shpi.advanced.model.structure.MaterialList;
import sa.shpi.advanced.model.structure.Section;
import sa.shpi.advanced.model.structure.SectionCategory;
import sa.shpi.advanced.model.structure.StructureUsedMaterials;
import sa.shpi.advanced.pdf.PdfGenerator;
import sa.shpi.advanced.visual.content.RoomDetailForm;
import sa.shpi.advanced.visual.structure.MaterialDetailForm;
import sa.shpi.advanced.visual.shelf.ShelfDetailForm;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;
import javax.swing.JLabel;
import java.awt.Font;
import java.awt.Component;
import javax.swing.Box;
import sa.shpi.advanced.model.shelf.Shelf;
import sa.shpi.advanced.model.shelf.ShelfSubType;

/**
 * Main landing form for the advanced model.
 *
 * @author Kyle
 *
 */
public class AdvancedMainForm {

    private JFrame frame;
    JTree tree = new JTree();

    JMenuItem mntmWarehouse = new JMenuItem(Strings.WAREHOUSE.toString()
            + "...");
    JMenuItem mntmStructureItem = new JMenuItem(
            Strings.STRUCTURE_ITEM.toString() + "...");
    JMenuItem mntmAffectedRoom = new JMenuItem(Strings.AFFECTED_ROOM.toString()
            + "...");
    JMenuItem mntmAffectedShelf = new JMenuItem(
            Strings.AFFECTED_SHELF.toString() + "...");

    private AdvancedTool tool;
    private final JScrollPane scrollPane = new JScrollPane();
    private final JMenu mnNew_1 = new JMenu(Strings.NUEVO.toString());
    private final JMenuItem resetButton = new JMenuItem(
            Strings.RESET.toString());
    private final JMenuItem exitButton = new JMenuItem(Strings.EXIT.toString());
    private final JPanel panel_1 = new JPanel();
    private final JButton newStructureMaterialButton = new JButton(
            Strings.NUEVO.toString() + " " + Strings.STRUCTURE_ITEM.toString()
            + "...");
    private final JButton newAffectedRoomButton = new JButton(
            Strings.NUEVA.toString() + " " + Strings.AFFECTED_ROOM.toString()
            + "...");
    private final JButton newAffectedShelfButton = new JButton(
            Strings.NUEVO.toString() + " " + Strings.AFFECTED_SHELF.toString()
            + "...");
    private final JMenu mnGen = new JMenu(Strings.REPORT.toString());
    private final JMenuItem generateReportButton = new JMenuItem(
            Strings.GENERATE_REPORT.toString());
    private final JPanel panel_2 = new JPanel();
    private final JLabel emissionsTotalLabel = new JLabel("0.00 kg CO2");
    private final JButton newWarehouseButton = new JButton(
            Strings.NEW_WAREHOUSE.toString() + "...");
    private final SHPIImagePanel shpiSplashPanel = new SHPIImagePanel();
    private final Component horizontalStrut = Box.createHorizontalStrut(20);
    private final JMenu mnHelp = new JMenu(Strings.HELP.toString());
    private final JMenuItem mntmAbout = new JMenuItem(Strings.ABOUT.toString()
            + "...");

    /**
     * @wbp.parser.constructor
     */
    public AdvancedMainForm() {
        initialize();
    }

    public AdvancedMainForm(AdvancedTool tool) {
        this.tool = tool;
        initialize();
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        frame = new JFrame();
        frame.setResizable(false);
        frame.setTitle(Strings.WAREHOUSE_EMISSIONS_CALCULATOR.toString());
        frame.setBounds(100, 100, 680, 500);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        GridBagLayout gridBagLayout = new GridBagLayout();
        gridBagLayout.columnWidths = new int[]{297, 234, 0};
        gridBagLayout.rowHeights = new int[]{376, 0};
        gridBagLayout.columnWeights = new double[]{0.0, 1.0, Double.MIN_VALUE};
        gridBagLayout.rowWeights = new double[]{1.0, Double.MIN_VALUE};
        frame.getContentPane().setLayout(gridBagLayout);

        GridBagConstraints gbc_scrollPane = new GridBagConstraints();
        gbc_scrollPane.ipadx = 385;
        gbc_scrollPane.ipady = 500;
        gbc_scrollPane.insets = new Insets(0, 0, 0, 5);
        gbc_scrollPane.gridx = 0;
        gbc_scrollPane.gridy = 0;
        frame.getContentPane().add(scrollPane, gbc_scrollPane);
        scrollPane.setViewportView(tree);

        DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) tree
                .getCellRenderer();
        renderer.setLeafIcon(null);
        renderer.setClosedIcon(null);
        renderer.setOpenIcon(null);

        tree.setCellRenderer(renderer);
        tree.setRootVisible(true);
        tree.setModel(new DefaultTreeModel(new DefaultMutableTreeNode(
                Strings.INSTRUCTION_TREE) {
                    {
                    }
                }));

        scrollPane.setColumnHeaderView(horizontalStrut);

        JPanel panel = new JPanel();
        GridBagConstraints gbc_panel = new GridBagConstraints();
        gbc_panel.fill = GridBagConstraints.BOTH;
        gbc_panel.gridx = 1;
        gbc_panel.gridy = 0;
        frame.getContentPane().add(panel, gbc_panel);
        GridBagLayout gbl_panel = new GridBagLayout();
        gbl_panel.columnWidths = new int[]{0, 0};
        gbl_panel.rowHeights = new int[]{0, 0, 0, 0};
        gbl_panel.columnWeights = new double[]{1.0, Double.MIN_VALUE};
        gbl_panel.rowWeights = new double[]{0.0, 0.0, 1.0, Double.MIN_VALUE};
        panel.setLayout(gbl_panel);

        GridBagConstraints gbc_panel_1 = new GridBagConstraints();
        gbc_panel_1.insets = new Insets(0, 0, 5, 0);
        gbc_panel_1.fill = GridBagConstraints.BOTH;
        gbc_panel_1.gridx = 0;
        gbc_panel_1.gridy = 0;
        panel_1.setBorder(new TitledBorder(null, Strings.CREATE.toString(),
                TitledBorder.LEADING, TitledBorder.TOP, null, null));
        panel.add(panel_1, gbc_panel_1);
        GridBagLayout gbl_panel_1 = new GridBagLayout();
        gbl_panel_1.columnWidths = new int[]{0, 0};
        gbl_panel_1.rowHeights = new int[]{0, 0, 0, 0};
        gbl_panel_1.columnWeights = new double[]{1.0, Double.MIN_VALUE};
        gbl_panel_1.rowWeights = new double[]{0.0, 0.0, 0.0, Double.MIN_VALUE};
        panel_1.setLayout(gbl_panel_1);

        GridBagConstraints gbc_newWarehouseButton = new GridBagConstraints();
        gbc_newWarehouseButton.insets = new Insets(0, 0, 5, 0);
        gbc_newWarehouseButton.gridx = 0;
        gbc_newWarehouseButton.gridy = 0;
        newWarehouseButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mntmWarehouse.doClick();
            }
        });
        panel_1.add(newWarehouseButton, gbc_newWarehouseButton);

        GridBagConstraints gbc_newStructureMaterialButton = new GridBagConstraints();
        gbc_newStructureMaterialButton.insets = new Insets(0, 0, 5, 0);
        gbc_newStructureMaterialButton.gridx = 0;
        gbc_newStructureMaterialButton.gridy = 1;
        newStructureMaterialButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mntmStructureItem.doClick();
            }
        });
        newStructureMaterialButton.setEnabled(false);
        panel_1.add(newStructureMaterialButton, gbc_newStructureMaterialButton);

        GridBagConstraints gbc_newAffectedRoomButton = new GridBagConstraints();
        gbc_newAffectedRoomButton.insets = new Insets (0, 0, 5, 0);
        gbc_newAffectedRoomButton.gridx = 0;
        gbc_newAffectedRoomButton.gridy = 2;
        newAffectedRoomButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mntmAffectedRoom.doClick();
            }
        });
        newAffectedRoomButton.setEnabled(false);
        panel_1.add(newAffectedRoomButton, gbc_newAffectedRoomButton);
        
        GridBagConstraints gbc_newAffectedShelfButton = new GridBagConstraints();
        gbc_newAffectedShelfButton.gridx = 0;
        gbc_newAffectedShelfButton.gridy = 3;
        newAffectedShelfButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mntmAffectedShelf.doClick();
            }
        });
        newAffectedShelfButton.setEnabled(false);
        panel_1.add(newAffectedShelfButton, gbc_newAffectedShelfButton);

        GridBagConstraints gbc_panel_2 = new GridBagConstraints();
        gbc_panel_2.insets = new Insets(0, 0, 5, 0);
        gbc_panel_2.fill = GridBagConstraints.BOTH;
        gbc_panel_2.gridx = 0;
        gbc_panel_2.gridy = 1;
        panel_2.setBorder(new TitledBorder(null, Strings.EMISSIONS_TOTAL
                .toString(), TitledBorder.LEADING, TitledBorder.TOP, null, null));
        panel.add(panel_2, gbc_panel_2);
        emissionsTotalLabel.setFont(new Font("Tahoma", Font.PLAIN, 18));

        panel_2.add(emissionsTotalLabel);

        GridBagConstraints gbc_shpiSplashPanel = new GridBagConstraints();
        gbc_shpiSplashPanel.fill = GridBagConstraints.BOTH;
        gbc_shpiSplashPanel.gridx = 0;
        gbc_shpiSplashPanel.gridy = 2;
        panel.add(shpiSplashPanel, gbc_shpiSplashPanel);

        JMenuBar menuBar = new JMenuBar();
        frame.setJMenuBar(menuBar);

        JMenu mnNew = new JMenu(Strings.FILE.toString());
        menuBar.add(mnNew);

        mnNew.add(mnNew_1);
        mnNew_1.add(mntmWarehouse);
        
        mnNew_1.add(mntmStructureItem);
        mntmStructureItem.setEnabled(false);
        
        mnNew_1.add(mntmAffectedRoom);
        mntmAffectedRoom.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                RoomDetailForm newRoomForm = new RoomDetailForm(
                        AdvancedMainForm.this);
            }
        });
        mntmAffectedRoom.setEnabled(false);
        
        mnNew_1.add(mntmAffectedShelf);
        mntmAffectedShelf.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ShelfDetailForm ShelfForm = new ShelfDetailForm(AdvancedMainForm.this);
            }
        });
        mntmAffectedShelf.setEnabled(false);
        
        resetButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tool.setWarehouse(new Warehouse());

                mntmWarehouse.setEnabled(true);
                mntmStructureItem.setEnabled(false);
                mntmAffectedRoom.setEnabled(false);
                mntmAffectedShelf.setEnabled(false);
                generateReportButton.setEnabled(false);

                newWarehouseButton.setEnabled(true);
                newStructureMaterialButton.setEnabled(false);
                newAffectedRoomButton.setEnabled(false);
                newAffectedShelfButton.setEnabled(false);

                resetAll();
            }
        });

        mnNew.add(resetButton);
        exitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                frame.setVisible(false);
                frame.dispose();
            }
        });

        mnNew.add(exitButton);

        menuBar.add(mnGen);
        generateReportButton.setEnabled(false);
        generateReportButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                PdfGenerator generate = new PdfGenerator(tool.getWarehouse());
                JOptionPane.showMessageDialog(frame,
                        Strings.SUCCESSFUL_REPORT.toString());
            }
        });

        mnGen.add(generateReportButton);

        menuBar.add(mnHelp);
        mntmAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                AboutForm aboutForm = new AboutForm();
            }
        });

        mnHelp.add(mntmAbout);
        mntmStructureItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                MaterialDetailForm newMatForm = new MaterialDetailForm(
                        AdvancedMainForm.this);
            }
        });

        mntmWarehouse.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                WarehouseDetailForm newIndForm = new WarehouseDetailForm(
                        AdvancedMainForm.this);
            }
        });

    }

    public void addWarehouseItem(Warehouse ware) {
        // update visual components
        mntmWarehouse.setEnabled(false);
        mntmStructureItem.setEnabled(true);
        mntmAffectedRoom.setEnabled(true);
        mntmAffectedShelf.setEnabled(true);
        generateReportButton.setEnabled(true);

        newWarehouseButton.setEnabled(false);
        newStructureMaterialButton.setEnabled(true);
        newAffectedRoomButton.setEnabled(true);
        newAffectedShelfButton.setEnabled(true);

        //Set the new industrial model:
        tool.setWarehouse(ware);

        refreshAll();
    }

    public void addStructureItem(SectionCategory section, StructureUsedMaterials uMaterial) {
        //Add a structural material to the chosen area:
        tool.getWarehouse().getSection(section).addMaterial(uMaterial);
        refreshAll();
    }

    public void addRoom(Room room) {
        //Add a room to the building:
        tool.getWarehouse().addRoom(room);
        refreshAll();
    }
    
    public void addShelf (Shelf shelf) {
        //Add a shelf to the building:
        tool.getWarehouse().addShelf(shelf);
        refreshAll();
    }

    public void resetAll() {
        //Remove all nodes in tree and update the emissions model:
        tree.removeAll();
        refreshEmissionsLabel();
        DefaultMutableTreeNode top = new DefaultMutableTreeNode(
                Strings.INSTRUCTION_TREE.toString());
        tree.setModel(new DefaultTreeModel(top));
    }

    public void refreshAll() {
        refreshEmissionsLabel();
        refreshTree();
    }

    public void refreshEmissionsLabel() {
        emissionsTotalLabel.setText(String.format("%.2f kg CO2", tool
                .getWarehouse().getTotalEmissions()));
    }

    public void refreshTree() {
        //Dynamically create the tree for the industrial building.
        DefaultMutableTreeNode top = new DefaultMutableTreeNode(
                tool.getWarehouse());

        DefaultMutableTreeNode structure = new DefaultMutableTreeNode(
                Strings.STRUCTURE.toString()
                + String.format(" (%.2f kg CO2)", tool.getWarehouse().getStructureEmissions())
        );

        DefaultMutableTreeNode externalWalls = new DefaultMutableTreeNode(tool
                .getWarehouse().getExternalWalls());
        DefaultMutableTreeNode flooring = new DefaultMutableTreeNode(tool
                .getWarehouse().getFlooring());
        DefaultMutableTreeNode roofing = new DefaultMutableTreeNode(tool
                .getWarehouse().getRoofing());

        addTreeMaterials(tool.getWarehouse().getExternalWalls(),
                externalWalls);
        addTreeMaterials(tool.getWarehouse().getFlooring(), flooring);
        addTreeMaterials(tool.getWarehouse().getRoofing(), roofing);

        structure.add(externalWalls);
        structure.add(flooring);
        structure.add(roofing);

        DefaultMutableTreeNode content = new DefaultMutableTreeNode(
                Strings.CONTENT.toString()
                + String.format(" (%.2f kg CO2)", tool.getWarehouse()
                        .getContentEmissions()));
        addRooms(tool.getWarehouse().getRooms(), content);
        
        DefaultMutableTreeNode shelves = new DefaultMutableTreeNode(
            Strings.SHELF.toString() + String.format(" (%.2f kg CO2)", tool.getWarehouse().getShelfEmissions()));
        addShelves(tool.getWarehouse().getShelves(), shelves);

        top.add(structure);
        top.add(content);
        top.add(shelves);

        TreeModel treeModel = new DefaultTreeModel(top);

        tree.removeAll();
        tree.setModel(treeModel);

        for (int count = 0; count < tree.getRowCount(); count++) {
            tree.expandRow(count);
        }
    }

    private void addRooms(ArrayList<Room> rooms,
            DefaultMutableTreeNode contentNode) {
        /*
         * Add room nodes to the content node.
         * Loop through each room.
         */
        if (rooms.size() > 0) {
            for (int count = 0; count < rooms.size(); count++) {
                //Create a new node:
                DefaultMutableTreeNode roomNode = new DefaultMutableTreeNode(
                        rooms.get(count));
                //Add furnishings to the node:
                addFurnishings(rooms.get(count).getFurnishingList(), roomNode);
                //Add room to the content node:
                contentNode.add(roomNode);
            }
        } else {
            //If no rooms, add 'none' string:
            contentNode.add(new DefaultMutableTreeNode("("
                    + Strings.NONE.toString() + ")"));
        }
    }

    private void addFurnishings(FurnishingList furnishingList,
            DefaultMutableTreeNode roomNode) {
        //For each furnishing in the room:
        if (furnishingList.size() > 0) {
            for (int count = 0; count < furnishingList.size(); count++) {
                DefaultMutableTreeNode furnishingNode = new DefaultMutableTreeNode(
                        furnishingList.get(count));
                //Add a furnishing node:
                roomNode.add(furnishingNode);
            }
        } else {
            //If no furnishings, add 'none' string:
            roomNode.add(new DefaultMutableTreeNode("("
                    + Strings.NONE.toString() + ")"));
        }
    }
    
    private void addShelves(ArrayList<Shelf> shelves,
            DefaultMutableTreeNode shelfContentNode) {
        /*
         * Add shelf nodes to the content node.
         * Loop through each shelf.
         */
        if (shelves.size() > 0) {
            for (int count = 0; count < shelves.size(); count++) {
                //Create a new node:
                DefaultMutableTreeNode shelfNode = new DefaultMutableTreeNode(
                        shelves.get(count));
                //Add subtype to the node:
                addShelfSubType(shelves.get(count).getShelfSubType(), shelves.get(count).getShelfQuantity(), 
                        Double.parseDouble((String.format("%.2f", shelves.get(count).getUsageFraction() * 100))),
                        shelves.get(count).getDamage(), shelfNode);
                //Add room to the content node:
                shelfContentNode.add(shelfNode);
            }
        } else {
            //If no shelves, add 'none' string:
            shelfContentNode.add(new DefaultMutableTreeNode("("
                    + Strings.NONE.toString() + ")"));
        }
    }
    
    private void addShelfSubType(ShelfSubType shelfsubtype, int shelfQuantity, double usage, 
            int damage, DefaultMutableTreeNode roomNode) {
                
                DefaultMutableTreeNode furnishingNode = new DefaultMutableTreeNode(
                        shelfsubtype.toString() + ", " + shelfQuantity + "x, " +
                        usage + "% " + Strings.USAGE.toString() + ", "
                        + damage + "% " + Strings.DAMAGE.toString());
                //Add a furnishing node:
                roomNode.add(furnishingNode);   
    }

    private void addTreeMaterials(Section section,
            DefaultMutableTreeNode sectionNode) {
        if (section.getMaterialList().size() > 0) {
            MaterialList mList = section.getMaterialList();

            //For each material in the list:
            for (int count = 0; count < mList.size(); count++) {
                DefaultMutableTreeNode matNode = new DefaultMutableTreeNode(
                        mList.get(count));
                //Add the material to the structural location:
                sectionNode.add(matNode);
            }
        } else {
            //If no materials, add 'none' string:
            sectionNode.add(new DefaultMutableTreeNode("("
                    + Strings.NONE.toString() + ")"));
        }
    }

    public AdvancedTool getTool() {
        return tool;
    }

    public JFrame getFrame() {
        return frame;
    }
}
