/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.visual;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import javax.swing.JLabel;
import java.awt.GridBagConstraints;
import javax.swing.JTextField;
import java.awt.Insets;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;
import javax.swing.JButton;
import java.awt.Component;
import javax.swing.Box;
import sa.shpi.advanced.model.Strings;
import sa.shpi.advanced.model.Warehouse;
import sa.shpi.advanced.model.content.Room;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Dialog.ModalityType;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import javax.swing.ImageIcon;

/**
 * When creating a new building, this form allows the user to input the total
 * area and area affected, as well as the type of warehouse.
 *
 * @author Kyle
 *
 */
public class WarehouseDetailForm extends JDialog {

    private static final long serialVersionUID = 1L;

    private AdvancedMainForm parent;
    private JTextField totalAreaTextField;
    private JTextField areaBurnedTextField;
    private String doubleregex = "^[1-9][0-9]*[.][0-9]*$";
    private String intregex = "^[1-9][0-9]*$";
    
    public WarehouseDetailForm(AdvancedMainForm parent) {
        super(parent.getFrame());
        this.parent = parent;
        initialize();
    }
   
    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        setTitle(Strings.NEW_WAREHOUSE.toString());
        setLocationRelativeTo(parent.getFrame());
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setModalityType(ModalityType.APPLICATION_MODAL);
        setBounds(100, 100, 300, 240);
        setResizable(false);

        JPanel panel = new JPanel();
        getContentPane().add(panel, BorderLayout.CENTER);
        GridBagLayout gbl_panel = new GridBagLayout();
        gbl_panel.columnWidths = new int[]{70, 100, 0, 0};
        gbl_panel.rowHeights = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0};
        gbl_panel.columnWeights = new double[]{0.0, 0.0, 0.0,
            Double.MIN_VALUE};
        gbl_panel.rowWeights = new double[]{0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 1.0, Double.MIN_VALUE};
        panel.setLayout(gbl_panel);

        Component verticalStrut = Box.createVerticalStrut(20);
        GridBagConstraints gbc_verticalStrut = new GridBagConstraints();
        gbc_verticalStrut.insets = new Insets(0, 0, 5, 5);
        gbc_verticalStrut.gridx = 1;
        gbc_verticalStrut.gridy = 0;
        panel.add(verticalStrut, gbc_verticalStrut);

        JLabel lblNewLabel = new JLabel(Strings.AREA_TOTAL.toString());
        GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
        gbc_lblNewLabel.anchor = GridBagConstraints.WEST;
        gbc_lblNewLabel.insets = new Insets(0, 0, 5, 5);
        gbc_lblNewLabel.gridx = 1;
        gbc_lblNewLabel.gridy = 1;
        panel.add(lblNewLabel, gbc_lblNewLabel);

        Component horizontalStrut = Box.createHorizontalStrut(20);
        GridBagConstraints gbc_horizontalStrut = new GridBagConstraints();
        gbc_horizontalStrut.insets = new Insets(0, 0, 5, 5);
        gbc_horizontalStrut.gridx = 0;
        gbc_horizontalStrut.gridy = 2;
        panel.add(horizontalStrut, gbc_horizontalStrut);

        totalAreaTextField = new JTextField();
        totalAreaTextField.setHorizontalAlignment(SwingConstants.RIGHT);
        totalAreaTextField.setText("1.00");
        totalAreaTextField.addFocusListener(new FocusListener() {

            @Override
            public void focusGained(FocusEvent e) {
                totalAreaTextField.selectAll();
            }

            @Override
            public void focusLost(FocusEvent e) {
                if (!totalAreaTextField.getText().matches(doubleregex)) {
                    if (totalAreaTextField.getText().matches(intregex)) {
                        String value = totalAreaTextField.getText();
                        totalAreaTextField.setText(value + ".00");
                    } else {
                        totalAreaTextField.setText("1.00");
                        ImageIcon shpi = new ImageIcon(WarehouseDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
                        JOptionPane.showMessageDialog(null, Strings.ERROR_BASE.toString() + Strings.AREA_TOTAL.toString(),
                                Strings.AREA_BURNED.toString(), JOptionPane.WARNING_MESSAGE, shpi);
                    }
                }
            }
            
        });
        GridBagConstraints gbc_totalAreaTextField = new GridBagConstraints();
        gbc_totalAreaTextField.insets = new Insets(0, 0, 5, 5);
        gbc_totalAreaTextField.fill = GridBagConstraints.HORIZONTAL;
        gbc_totalAreaTextField.gridx = 1;
        gbc_totalAreaTextField.gridy = 2;
        panel.add(totalAreaTextField, gbc_totalAreaTextField);
        totalAreaTextField.setColumns(10);

        Component verticalStrut_1 = Box.createVerticalStrut(20);
        GridBagConstraints gbc_verticalStrut_1 = new GridBagConstraints();
        gbc_verticalStrut_1.insets = new Insets(0, 0, 5, 5);
        gbc_verticalStrut_1.gridx = 1;
        gbc_verticalStrut_1.gridy = 3;
        panel.add(verticalStrut_1, gbc_verticalStrut_1);

        JLabel lblNewLabel_1 = new JLabel(Strings.AREA_BURNED.toString());
        GridBagConstraints gbc_lblNewLabel_1 = new GridBagConstraints();
        gbc_lblNewLabel_1.insets = new Insets(0, 0, 5, 5);
        gbc_lblNewLabel_1.anchor = GridBagConstraints.ABOVE_BASELINE_LEADING;
        gbc_lblNewLabel_1.gridx = 1;
        gbc_lblNewLabel_1.gridy = 4;
        panel.add(lblNewLabel_1, gbc_lblNewLabel_1);

        Component horizontalStrut_1 = Box.createHorizontalStrut(20);
        GridBagConstraints gbc_horizontalStrut_1 = new GridBagConstraints();
        gbc_horizontalStrut_1.insets = new Insets(0, 0, 5, 5);
        gbc_horizontalStrut_1.gridx = 0;
        gbc_horizontalStrut_1.gridy = 5;
        panel.add(horizontalStrut_1, gbc_horizontalStrut_1);

        areaBurnedTextField = new JTextField();
        areaBurnedTextField.setHorizontalAlignment(SwingConstants.RIGHT);
        areaBurnedTextField.setText("1.00");
        areaBurnedTextField.addFocusListener(new FocusListener() {

            @Override
            public void focusGained(FocusEvent e) {
                areaBurnedTextField.selectAll();
            }

            @Override
            public void focusLost(FocusEvent e) {
                if (!areaBurnedTextField.getText().matches(doubleregex)) {
                    if (areaBurnedTextField.getText().matches(intregex)) {
                        String value = areaBurnedTextField.getText();
                        areaBurnedTextField.setText(value + ".00");
                    } else {
                        areaBurnedTextField.setText("1.00");
                        ImageIcon shpi = new ImageIcon(WarehouseDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
                        JOptionPane.showMessageDialog(null, Strings.ERROR_BASE.toString() + Strings.AREA_BURNED.toString(),
                                Strings.AREA_BURNED.toString(), JOptionPane.WARNING_MESSAGE, shpi);
                    }
                }     
            }
            
        });
        GridBagConstraints gbc_areaBurnedTextField = new GridBagConstraints();
        gbc_areaBurnedTextField.insets = new Insets(0, 0, 5, 5);
        gbc_areaBurnedTextField.fill = GridBagConstraints.HORIZONTAL;
        gbc_areaBurnedTextField.gridx = 1;
        gbc_areaBurnedTextField.gridy = 5;
        panel.add(areaBurnedTextField, gbc_areaBurnedTextField);
        areaBurnedTextField.setColumns(10);

        Component verticalStrut_2 = Box.createVerticalStrut(20);
        GridBagConstraints gbc_verticalStrut_2 = new GridBagConstraints();
        gbc_verticalStrut_2.insets = new Insets(0, 0, 5, 5);
        gbc_verticalStrut_2.gridx = 1;
        gbc_verticalStrut_2.gridy = 6;
        panel.add(verticalStrut_2, gbc_verticalStrut_2);

        Component horizontalStrut_3 = Box.createHorizontalStrut(20);
        GridBagConstraints gbc_horizontalStrut_3 = new GridBagConstraints();
        gbc_horizontalStrut_3.insets = new Insets(0, 0, 0, 5);
        gbc_horizontalStrut_3.gridx = 0;
        gbc_horizontalStrut_3.gridy = 11;
        panel.add(horizontalStrut_3, gbc_horizontalStrut_3);

        //Cancel button to dispose current form
        JButton cancelButton = new JButton(Strings.CANCEL.toString());
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
                dispose();
            }
        });
        GridBagConstraints gbc_cancelButton = new GridBagConstraints();
        gbc_cancelButton.anchor = GridBagConstraints.SOUTH;
        gbc_cancelButton.insets = new Insets(0, 0, 0, 5);
        gbc_cancelButton.gridx = 1;
        gbc_cancelButton.gridy = 11;
        panel.add(cancelButton, gbc_cancelButton);
        
        //Add button to add current info to the structure.
        JButton addButton = new JButton(Strings.ADD.toString());
        addButton.addActionListener((ActionEvent e) -> {
            if (Double.parseDouble(totalAreaTextField.getText()) < Double.parseDouble(areaBurnedTextField.getText())){
                ImageIcon shpi = new ImageIcon(WarehouseDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
                JOptionPane.showMessageDialog(null, Strings.ERROR_BURNED_BIGGER, Strings.WAREHOUSE.toString(),
                        JOptionPane.WARNING_MESSAGE, shpi);
            } else {
                ImageIcon shpi = new ImageIcon(WarehouseDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
                int response = JOptionPane.showConfirmDialog(null, Strings.CONFIRM_DATA, Strings.CONFIRM.toString(), JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE, shpi);
                if (response == JOptionPane.CANCEL_OPTION) {
                    setVisible(true);
                } else if (response == JOptionPane.OK_OPTION) {
                    Warehouse ware = new Warehouse();
                    ware.setAreaTotal(Double.parseDouble(totalAreaTextField.getText()));
                    ware.setAreaBurned(Double.parseDouble(areaBurnedTextField.getText()));
                    parent.addWarehouseItem(ware);
                    setVisible(false);
                    dispose();
                }
            }
        });
        //Sets the default highlighted button:
        getRootPane().setDefaultButton(addButton);

        GridBagConstraints gbc_addButton = new GridBagConstraints();
        gbc_addButton.anchor = GridBagConstraints.SOUTH;
        gbc_addButton.insets = new Insets(0, 0, 0, 5);
        gbc_addButton.gridx = 2;
        gbc_addButton.gridy = 11;
        panel.add(addButton, gbc_addButton);

        this.setVisible(true);

        Component horizontalStrut_4 = Box.createHorizontalStrut(20);
        GridBagConstraints gbc_horizontalStrut_4 = new GridBagConstraints();
        gbc_horizontalStrut_4.insets = new Insets(0, 0, 0, 5);
        gbc_horizontalStrut_4.gridx = 0;
        gbc_horizontalStrut_4.gridy = 11;
        panel.add(horizontalStrut_4, gbc_horizontalStrut_4);

    }

}
