/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.visual.shelf;

import sa.shpi.advanced.model.Strings;
import sa.shpi.advanced.model.shelf.Shelf;
import sa.shpi.advanced.model.shelf.ShelfType;
import sa.shpi.advanced.visual.AdvancedMainForm;
import sa.shpi.advanced.model.Warehouse;
import sa.shpi.advanced.model.shelf.ChemicalSubType;
import sa.shpi.advanced.model.shelf.FoodSubType;
import sa.shpi.advanced.model.shelf.MetalSubType;
import sa.shpi.advanced.model.shelf.PaperSubType;
import sa.shpi.advanced.model.shelf.ShelfBaseType;
import sa.shpi.advanced.model.shelf.ShelfSubType;
import sa.shpi.advanced.model.shelf.TextileSubType;
import sa.shpi.advanced.model.shelf.WoodSubType;

import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.Dialog.ModalityType;
import java.awt.Window.Type;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;
import javax.swing.SwingConstants;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JSeparator;
import sa.shpi.advanced.model.shelf.ElectronicSubType;
import sa.shpi.advanced.model.shelf.FuelSubType;
import sa.shpi.advanced.model.shelf.PlasticSubType;

/**
 * Form for a room that allows a user to add multiple furniture items to a room.
 * Also contains logical steps to validate that the room area does not exceed the
 * structure area, and that if a room uses a preset default, that it does not
 * also use individual furnishing items.
 *
 * @author Kyle
 *
 */
public class ShelfDetailForm extends JDialog {

    private static final long serialVersionUID = 1L;
    
    private JFrame frame = new JFrame();
    private final JPanel contentPanel = new JPanel();
    private JLabel damageLabel = new JLabel("100%");
    private JSlider damageSlider = new JSlider();
    private final JPanel panel1 = new JPanel();
    private JTextField shelfWidthText = new JTextField("0.92");
    private JTextField shelfLengthText = new JTextField("2.88");
    private JTextField shelfHeightText = new JTextField("4.31");
    private final JPanel panel3 = new JPanel();
    private JComboBox shelfTypeComboBox = new JComboBox();
    private JButton addButton = new JButton(Strings.ADD.toString());
    private final JPanel panel4 = new JPanel();
    private JTextField shelfQuantity = new JTextField("1");
    private JLabel usageLabel = new JLabel("100%");
    private JSlider usageSlider = new JSlider();
    private final JPanel panel5 = new JPanel();
    private JTextField shelfLevels = new JTextField("1");
    private JComboBox chemicalSubType = new JComboBox();
    private JComboBox foodSubType = new JComboBox();
    private JComboBox metalSubType = new JComboBox();
    private JComboBox textileSubType = new JComboBox();
    private JComboBox paperSubType = new JComboBox();
    private JComboBox woodSubType = new JComboBox();
    private JComboBox electronicSubType = new JComboBox();
    private JComboBox plasticSubType = new JComboBox();
    private JComboBox fuelSubType = new JComboBox();
    private JComboBox nullSubType = new JComboBox();
    private final JPanel panel6 = new JPanel();
    private JComboBox shelfBase = new JComboBox();
    private JButton customDim = new JButton();
    private JButton standardDim = new JButton();
    private final JButton levelHelp = new JButton("?");
    private final JButton heightHelp = new JButton("?");
    private final JButton lengthHelp = new JButton("?");
    private final JButton widthHelp = new JButton("?");
    private final JButton baseHelp = new JButton("?");
    private final JButton quantityHelp = new JButton("?");
    private final JButton shelfTypeHelp = new JButton("?");
    private final JButton subTypeHelp = new JButton("?");
    private final JButton usageHelp = new JButton("?");
    private final JButton damageHelp = new JButton("?");
    private JSlider heightusageslider = new JSlider();
    private JLabel heightUsageLabel = new JLabel();
    private final JButton heightUsageHelp = new JButton("?");
    
    //Validation strings for text input.
    private String doubleregex = "^[0-9]*[.][0-9]*$";
    private String intregex = "^[1-9][0-9]*$";

    private Shelf shelf;
    private Warehouse ware;

    private AdvancedMainForm parent;
    

    /**
     * Create the dialog.
     * 
     * @param parent
     */
    public ShelfDetailForm(AdvancedMainForm parent) {
        
        super(parent.getFrame());

        shelf = new Shelf();
        
        //Set initial value to dummy null
        nullSubType.setEnabled(false);
        
        //Set defualt values for shelf dimensions.
        shelf.setShelfWidth(0.92);
        shelf.setShelfLength(2.88);
        shelf.setShelfHeight(4.31);
        shelf.setShelfArea(2.88 * 0.92);
        shelf.setShelfVolume(2.88 * 0.92 * 4.31);
        shelf.setShelfLevels(1);
        shelf.setShelfQuantity(1);
        
        setTitle(Strings.SHELF.toString());
        setType(Type.UTILITY);
        setResizable(false);
        setModalityType(ModalityType.APPLICATION_MODAL);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        this.parent = parent;
        
        setBounds(100, 100, 1100, 500);
        getContentPane().setLayout(new BorderLayout());
        contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
        getContentPane().add(contentPanel, BorderLayout.CENTER);
        GridBagLayout gbl_contentPanel = new GridBagLayout();
        contentPanel.setLayout(gbl_contentPanel);
        {
            {
                JPanel panel = new JPanel();
                panel.setBorder(new TitledBorder(null,
                    Strings.SHELF_TYPE.toString(), TitledBorder.LEADING,
                    TitledBorder.TOP, null, null));
                GridBagConstraints gbc_panel = new GridBagConstraints();
                gbc_panel.insets = new Insets(0, 0, 5, 0);
                gbc_panel.fill = GridBagConstraints.BOTH;
                gbc_panel.gridx = 0;
                gbc_panel.gridy = 0;
                contentPanel.add(panel, gbc_panel);
                GridBagLayout gbl_panel = new GridBagLayout();
                gbl_panel.columnWidths = new int[]{1, 2, 1};
                gbl_panel.rowHeights = new int[]{0, 0};
                gbl_panel.columnWeights = new double[]{1.0, Double.MIN_VALUE};
                gbl_panel.rowWeights = new double[]{0.0, Double.MIN_VALUE};
                panel.setLayout(gbl_panel);
                
                {
                    //Combo box to input type of shelf.
                    shelfTypeComboBox.addActionListener((ActionEvent e) -> {
                        //Hide null dummy combo box
                        nullSubType.setVisible(false);
                        //Set ShelfType to the selection.
                        shelf.setShelfType((ShelfType) shelfTypeComboBox.getSelectedItem());
                        //Depending on selection, show appropriate SubType combo box.
                        //Set initial value to <null>
                        if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.CHEM)){
                            chemicalSubType.setVisible(true);
                            foodSubType.setVisible(false);
                            woodSubType.setVisible(false);
                            metalSubType.setVisible(false);
                            textileSubType.setVisible(false);
                            paperSubType.setVisible(false);
                            electronicSubType.setVisible(false);
                            fuelSubType.setVisible(false);
                            plasticSubType.setVisible(false);
                            chemicalSubType.setSelectedItem(null);
                        } else if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.FOOD)){
                            foodSubType.setEnabled(true);
                            chemicalSubType.setVisible(false);
                            foodSubType.setVisible(true);
                            woodSubType.setVisible(false);
                            metalSubType.setVisible(false);
                            textileSubType.setVisible(false);
                            paperSubType.setVisible(false);
                            electronicSubType.setVisible(false);
                            fuelSubType.setVisible(false);
                            plasticSubType.setVisible(false);
                            foodSubType.setSelectedItem(null);
                        } else if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.WOOD)){
                            chemicalSubType.setVisible(false);
                            foodSubType.setVisible(false);
                            woodSubType.setVisible(true);
                            metalSubType.setVisible(false);
                            textileSubType.setVisible(false);
                            paperSubType.setVisible(false);
                            electronicSubType.setVisible(false);
                            fuelSubType.setVisible(false);
                            plasticSubType.setVisible(false);
                            woodSubType.setSelectedItem(null);
                        } else if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.METAL)){
                            chemicalSubType.setVisible(false);
                            foodSubType.setVisible(false);
                            woodSubType.setVisible(false);
                            metalSubType.setVisible(true);
                            textileSubType.setVisible(false);
                            paperSubType.setVisible(false);
                            electronicSubType.setVisible(false);
                            fuelSubType.setVisible(false);
                            plasticSubType.setVisible(false);
                            metalSubType.setSelectedItem(null);
                        } else if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.TEXTILE)){
                            chemicalSubType.setVisible(false);
                            foodSubType.setVisible(false);
                            woodSubType.setVisible(false);
                            metalSubType.setVisible(false);
                            textileSubType.setVisible(true);
                            paperSubType.setVisible(false);
                            electronicSubType.setVisible(false);
                            fuelSubType.setVisible(false);
                            plasticSubType.setVisible(false);
                            textileSubType.setSelectedItem(null);
                        } else if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.PAPER)){
                            chemicalSubType.setVisible(false);
                            foodSubType.setVisible(false);
                            woodSubType.setVisible(false);
                            metalSubType.setVisible(false);
                            textileSubType.setVisible(false);
                            paperSubType.setVisible(true);
                            electronicSubType.setVisible(false);
                            fuelSubType.setVisible(false);
                            plasticSubType.setVisible(false);
                            paperSubType.setSelectedItem(null);
                        } else if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.ELECTRONICS)) {
                            chemicalSubType.setVisible(false);
                            foodSubType.setVisible(false);
                            woodSubType.setVisible(false);
                            metalSubType.setVisible(false);
                            textileSubType.setVisible(false);
                            paperSubType.setVisible(false);
                            electronicSubType.setVisible(true);
                            fuelSubType.setVisible(false);
                            plasticSubType.setVisible(false);
                            electronicSubType.setSelectedItem(null);
                        } else if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.FUEL)) {
                            chemicalSubType.setVisible(false);
                            foodSubType.setVisible(false);
                            woodSubType.setVisible(false);
                            metalSubType.setVisible(false);
                            textileSubType.setVisible(false);
                            paperSubType.setVisible(false);
                            electronicSubType.setVisible(false);
                            fuelSubType.setVisible(true);
                            plasticSubType.setVisible(false);
                            fuelSubType.setSelectedItem(null);
                        } else if (shelfTypeComboBox.getSelectedItem().equals(ShelfType.PLASTIC)) {
                            chemicalSubType.setVisible(false);
                            foodSubType.setVisible(false);
                            woodSubType.setVisible(false);
                            metalSubType.setVisible(false);
                            textileSubType.setVisible(false);
                            paperSubType.setVisible(false);
                            electronicSubType.setVisible(false);
                            fuelSubType.setVisible(false);
                            plasticSubType.setVisible(true);
                            plasticSubType.setSelectedItem(null);
                        }
                    });
                    shelfTypeComboBox.setModel(new DefaultComboBoxModel(ShelfType.values()));
                    shelfTypeComboBox.setMaximumRowCount(40);
                    GridBagConstraints gbc_shelfTypeComboBox = new GridBagConstraints();
                    gbc_shelfTypeComboBox.fill = GridBagConstraints.HORIZONTAL;
                    gbc_shelfTypeComboBox.gridx = 0;
                    gbc_shelfTypeComboBox.gridy = 0;
                    panel.add(shelfTypeComboBox, gbc_shelfTypeComboBox);
                }
                {
                    //Help box for shelf type.
                    ImageIcon shpi = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
                    shelfTypeHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "La categoria de producto almacenado en la unidad del estante.",
                                Strings.SHELF_TYPE.toString(), JOptionPane.INFORMATION_MESSAGE, shpi);
                    });
                    GridBagConstraints gbc_shelfTypeHelp = new GridBagConstraints();
                    gbc_shelfTypeHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_shelfTypeHelp.gridx = 1;
                    gbc_shelfTypeHelp.gridy = 0;
                    panel.add(shelfTypeHelp, gbc_shelfTypeHelp);
                }
                {
                    //Dummy combo box to force initial list value to null
                    nullSubType.setSelectedItem(null);
                    GridBagConstraints gbc_nullSubType = new GridBagConstraints();
                    gbc_nullSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_nullSubType.gridx = 0;
                    gbc_nullSubType.gridy = 1;
                    panel.add(nullSubType, gbc_nullSubType);
                }
                {
                    //Combo box to input subtype of chemical.
                    chemicalSubType.addActionListener((ActionEvent e) -> {
                        //Enable shelf levels input
                        shelfLevels.setEnabled(true);
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (chemicalSubType.getSelectedItem() == null) {}
                        else {
                            if (chemicalSubType.getSelectedItem().toString().contains(Strings.POOL_CHEM.toString())) {
                                shelf.setShelfSubType(ShelfSubType.POOL_CHEM);
                            } else if (chemicalSubType.getSelectedItem().toString().contains(Strings.PAINT.toString())){
                                shelf.setShelfSubType(ShelfSubType.PAINT);
                            } else if (chemicalSubType.getSelectedItem().toString().contains(ShelfSubType.DEF_CHEM.toString())) {
                                shelf.setShelfSubType(ShelfSubType.DEF_CHEM);
                            } else if (chemicalSubType.getSelectedItem().toString().contains(ShelfSubType.ANTIFREEZE.toString())) {
                                shelf.setShelfSubType(ShelfSubType.ANTIFREEZE);
                            } else if (chemicalSubType.getSelectedItem().toString().contains(ShelfSubType.RODENT.toString())) {
                                shelf.setShelfSubType(ShelfSubType.RODENT);
                            } else if (chemicalSubType.getSelectedItem().toString().contains(ShelfSubType.CLEANER.toString())) {
                                shelf.setShelfSubType(ShelfSubType.CLEANER);
                            } else if (chemicalSubType.getSelectedItem().toString().contains(ShelfSubType.LAUNDRY.toString())) {
                                shelf.setShelfSubType(ShelfSubType.LAUNDRY);
                            }
                        }
                    });
                    chemicalSubType.setModel(new DefaultComboBoxModel(ChemicalSubType.values()));
                    GridBagConstraints gbc_chemicalSubType = new GridBagConstraints();
                    gbc_chemicalSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_chemicalSubType.gridx = 0;
                    gbc_chemicalSubType.gridy = 1;
                    panel.add(chemicalSubType, gbc_chemicalSubType);
                    chemicalSubType.setVisible(false);
                }
                {
                    //Combo box to input subtype of plastic.
                    plasticSubType.addActionListener((ActionEvent e) -> {
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (plasticSubType.getSelectedItem() == null) {}
                        else {
                            if (plasticSubType.getSelectedItem().toString().contains(Strings.PVC.toString())) {
                                shelf.setShelfSubType(ShelfSubType.PVC);
                            } else if (plasticSubType.getSelectedItem().toString().contains(Strings.PET.toString())) {
                                shelf.setShelfSubType(ShelfSubType.PET);
                            } else if (plasticSubType.getSelectedItem().toString().contains(Strings.PS.toString())) {
                                shelf.setShelfSubType(ShelfSubType.PS);
                            } else if (plasticSubType.getSelectedItem().toString().contains(Strings.HDPE.toString())) {
                                shelf.setShelfSubType(ShelfSubType.HDPE);
                            } else if (plasticSubType.getSelectedItem().toString().contains(Strings.LDPE.toString())) {
                                shelf.setShelfSubType(ShelfSubType.LDPE);
                            } else if (plasticSubType.getSelectedItem().toString().contains(Strings.PP.toString())) {
                                shelf.setShelfSubType(ShelfSubType.PP);
                            } else if (plasticSubType.getSelectedItem().toString().contains(Strings.PLASTIC.toString())) {
                                shelf.setShelfSubType(ShelfSubType.DEF_PLASTIC);
                            }
                        }
                    });
                    plasticSubType.setModel(new DefaultComboBoxModel(PlasticSubType.values()));
                    GridBagConstraints gbc_plasticSubType = new GridBagConstraints();
                    gbc_plasticSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_plasticSubType.gridx = 0;
                    gbc_plasticSubType.gridy = 1;
                    panel.add(plasticSubType, gbc_plasticSubType);
                    plasticSubType.setVisible(false);
                }
                {
                    //Help box for shelf sub type
                    ImageIcon shpi = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
                    subTypeHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "Un tipo más especifíco del producto almacenado en la unidad del estante.\n"
                                        + "Si hay más de un producto en la unidad del estante o un otro producto"
                                        + " que no está en la lista elija la ultima opción (la con astriscos [*]).",
                                Strings.SHELF_SUB.toString(), JOptionPane.INFORMATION_MESSAGE, shpi);
                    });
                    GridBagConstraints gbc_subTypeHelp = new GridBagConstraints();
                    gbc_subTypeHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_subTypeHelp.gridx = 1;
                    gbc_subTypeHelp.gridy = 1;
                    panel.add(subTypeHelp, gbc_subTypeHelp);
                }
                {
                    //Combo box to input subtype of fuel.
                    fuelSubType.addActionListener((ActionEvent e) -> {
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (fuelSubType.getSelectedItem() == null) {}
                        else {
                            if (fuelSubType.getSelectedItem().toString().contains(Strings.GASOLINE.toString())){
                                shelf.setShelfSubType(ShelfSubType.GASOLINE);
                            } else if (fuelSubType.getSelectedItem().toString().contains(Strings.NAT_GAS.toString())){
                                shelf.setShelfSubType(ShelfSubType.NAT_GAS);
                            } else if (fuelSubType.getSelectedItem().toString().contains(Strings.BIODIESEL.toString())){
                                shelf.setShelfSubType(ShelfSubType.BIODIESEL);
                            } else if (fuelSubType.getSelectedItem().toString().contains(Strings.BUNKER_CRUDE.toString())){
                                shelf.setShelfSubType(ShelfSubType.BUNKER_CRUDE);
                            } else if (fuelSubType.getSelectedItem().toString().contains(Strings.DIESEL.toString())){
                                shelf.setShelfSubType(ShelfSubType.DIESEL);
                            } else if (fuelSubType.getSelectedItem().toString().contains(Strings.LPG.toString())){
                                shelf.setShelfSubType(ShelfSubType.LPG);
                            } else if (fuelSubType.getSelectedItem().toString().contains(Strings.FUEL.toString())){
                                shelf.setShelfSubType(ShelfSubType.DEF_FUEL);
                            }
                        }
                    });
                    fuelSubType.setModel(new DefaultComboBoxModel(FuelSubType.values()));
                    GridBagConstraints gbc_chemicalSubType = new GridBagConstraints();
                    gbc_chemicalSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_chemicalSubType.gridx = 0;
                    gbc_chemicalSubType.gridy = 1;
                    panel.add(fuelSubType, gbc_chemicalSubType);
                    fuelSubType.setVisible(false);
                }
                {
                    //Combo box to input subtype of food.
                    foodSubType.addActionListener((ActionEvent e) -> {
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (foodSubType.getSelectedItem() == null) {}
                        else {
                            if (foodSubType.getSelectedItem().toString().contains(Strings.ALCOHOL.toString())) {
                                shelf.setShelfSubType(ShelfSubType.ALCOHOL);
                            } else if (foodSubType.getSelectedItem().toString().contains(Strings.GRAINS.toString())){
                                shelf.setShelfSubType(ShelfSubType.GRAINS);
                            } else if (foodSubType.getSelectedItem().toString().contains(Strings.PRODUCE.toString())){
                                shelf.setShelfSubType(ShelfSubType.PRODUCE);
                            } else if (foodSubType.getSelectedItem().toString().contains(Strings.MEAT.toString())){
                                shelf.setShelfSubType(ShelfSubType.MEAT);
                            } else if (foodSubType.getSelectedItem().toString().contains(Strings.DAIRY.toString())){
                                shelf.setShelfSubType(ShelfSubType.DAIRY);
                            } else if (foodSubType.getSelectedItem().toString().contains(Strings.CANNED.toString())){
                                shelf.setShelfSubType(ShelfSubType.CANNED);
                            } else if (foodSubType.getSelectedItem().toString().contains(Strings.FOOD.toString())){
                                shelf.setShelfSubType(ShelfSubType.DEF_FOOD);
                            }
                        }
                    });
                    foodSubType.setModel(new DefaultComboBoxModel(FoodSubType.values()));
                    GridBagConstraints gbc_foodSubType = new GridBagConstraints();
                    gbc_foodSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_foodSubType.gridx = 0;
                    gbc_foodSubType.gridy = 1;
                    panel.add(foodSubType, gbc_foodSubType);
                    foodSubType.setVisible(false);
                }
                {
                    //Combo box to input subtype of wood.
                    woodSubType.addActionListener((ActionEvent e) -> {
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (woodSubType.getSelectedItem() == null) {}
                        else {
                            if (woodSubType.getSelectedItem().toString().contains(Strings.SOFT_PRESS.toString())) {
                                shelf.setShelfSubType(ShelfSubType.SOFT_PRESS);
                            } else if (woodSubType.getSelectedItem().toString().contains(Strings.SOFT.toString())){
                                shelf.setShelfSubType(ShelfSubType.SOFT);
                            } else if (woodSubType.getSelectedItem().toString().contains(Strings.HARD_PRESS.toString())){
                                shelf.setShelfSubType(ShelfSubType.HARD_PRESS);
                            } else if (woodSubType.getSelectedItem().toString().contains(Strings.HARD.toString())){
                                shelf.setShelfSubType(ShelfSubType.HARD);
                            } else if (woodSubType.getSelectedItem().toString().contains(Strings.PLYWOOD_HARD.toString())){
                                shelf.setShelfSubType(ShelfSubType.PLYWOOD_HARD);
                            } else if (woodSubType.getSelectedItem().toString().contains(Strings.PLYWOOD_SOFT.toString())) {
                                shelf.setShelfSubType(ShelfSubType.PLYWOOD_SOFT);
                            } else if (woodSubType.getSelectedItem().toString().contains(Strings.HDF.toString())){
                                shelf.setShelfSubType(ShelfSubType.HDF);
                            } else if (woodSubType.getSelectedItem().toString().contains(Strings.MDF.toString())) {
                                shelf.setShelfSubType(ShelfSubType.MDF);
                            } else if (woodSubType.getSelectedItem().toString().contains(Strings.WOOD.toString())) {
                                shelf.setShelfSubType(ShelfSubType.DEF_WOOD);
                            }
                        }
                    });
                    woodSubType.setModel(new DefaultComboBoxModel(WoodSubType.values()));
                    GridBagConstraints gbc_woodSubType = new GridBagConstraints();
                    gbc_woodSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_woodSubType.gridx = 0;
                    gbc_woodSubType.gridy = 1;
                    panel.add(woodSubType, gbc_woodSubType);
                    woodSubType.setVisible(false);
                }
                {
                    //Combo box to input subtype of metal.
                    metalSubType.addActionListener((ActionEvent e) -> {
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (metalSubType.getSelectedItem() == null) {}
                        else {
                            if (metalSubType.getSelectedItem().toString().contains(Strings.STEEL.toString())) {
                                shelf.setShelfSubType(ShelfSubType.STEEL);
                            } else if (metalSubType.getSelectedItem().toString().contains(Strings.ALUMINUM.toString())){
                                shelf.setShelfSubType(ShelfSubType.ALUMINUM);
                            } else if (metalSubType.getSelectedItem().toString().contains(Strings.IRON.toString())){
                                shelf.setShelfSubType(ShelfSubType.IRON);
                            } else if (metalSubType.getSelectedItem().toString().contains(Strings.COPPER.toString())){
                                shelf.setShelfSubType(ShelfSubType.COPPER);
                            } else if (metalSubType.getSelectedItem().toString().contains(Strings.ALLOY.toString())) {
                                shelf.setShelfSubType(ShelfSubType.ALLOY);
                            } else if (metalSubType.getSelectedItem().toString().contains(Strings.METAL.toString())) {
                                shelf.setShelfSubType(ShelfSubType.DEF_METAL);
                            }
                        }
                    });
                    metalSubType.setModel(new DefaultComboBoxModel(MetalSubType.values()));
                    GridBagConstraints gbc_metalSubType = new GridBagConstraints();
                    gbc_metalSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_metalSubType.gridx = 0;
                    gbc_metalSubType.gridy = 1;
                    panel.add(metalSubType, gbc_metalSubType);
                    metalSubType.setVisible(false);
                }
                {
                    //Combo box to input subtype of textile.
                    textileSubType.addActionListener((ActionEvent e) -> {
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (textileSubType.getSelectedItem() == null) {}
                        else {
                            if (textileSubType.getSelectedItem().toString().contains(Strings.LEATHER.toString())) {
                                shelf.setShelfSubType(ShelfSubType.LEATHER);
                            } else if (textileSubType.getSelectedItem().toString().contains(Strings.COTTON.toString())){
                                shelf.setShelfSubType(ShelfSubType.COTTON);
                            } else if (textileSubType.getSelectedItem().toString().contains(Strings.CARPET.toString())){
                                shelf.setShelfSubType(ShelfSubType.CARPET);
                            } else if (textileSubType.getSelectedItem().toString().contains(Strings.POLYESTER.toString())){
                                shelf.setShelfSubType(ShelfSubType.POLYESTER);
                            } else if (textileSubType.getSelectedItem().toString().contains(Strings.WOOL.toString())){
                                shelf.setShelfSubType(ShelfSubType.WOOL);
                            } else if (textileSubType.getSelectedItem().toString().contains(Strings.VINYL.toString())){
                                shelf.setShelfSubType(ShelfSubType.VINYL);
                            } else if (textileSubType.getSelectedItem().toString().contains(Strings.TEXTILE.toString())) {
                                shelf.setShelfSubType(ShelfSubType.DEF_TEXTILE);
                            }
                        }
                    });
                    textileSubType.setModel(new DefaultComboBoxModel(TextileSubType.values()));
                    GridBagConstraints gbc_textileSubType = new GridBagConstraints();
                    gbc_textileSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_textileSubType.gridx = 0;
                    gbc_textileSubType.gridy = 1;
                    panel.add(textileSubType, gbc_textileSubType);
                    textileSubType.setVisible(false);
                }
                {
                    //Combo box to input subtype of paper.
                    paperSubType.addActionListener((ActionEvent e) -> {
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (paperSubType.getSelectedItem() == null) {}
                        else {
                            if (paperSubType.getSelectedItem().toString().contains(Strings.COMPUTER_PAPER.toString())) {
                                shelf.setShelfSubType(ShelfSubType.COMPUTER_PAPER);
                            } else if (paperSubType.getSelectedItem().toString().contains(Strings.NEWSPAPER.toString())){
                                shelf.setShelfSubType(ShelfSubType.NEWSPAPER);
                            } else if (paperSubType.getSelectedItem().toString().contains(Strings.BATHROOM_PAPER.toString())){
                                shelf.setShelfSubType(ShelfSubType.BATHROOM_PAPER);
                            } else if (paperSubType.getSelectedItem().toString().contains(Strings.CARDBOARD.toString())){
                                shelf.setShelfSubType(ShelfSubType.CARDBOARD);
                            } else if (paperSubType.getSelectedItem().toString().contains(Strings.PAPER.toString())) {
                                shelf.setShelfSubType(ShelfSubType.DEF_PAPER);
                            }
                        }
                    });
                    paperSubType.setModel(new DefaultComboBoxModel(PaperSubType.values()));
                    GridBagConstraints gbc_paperSubType = new GridBagConstraints();
                    gbc_paperSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_paperSubType.gridx = 0;
                    gbc_paperSubType.gridy = 1;
                    panel.add(paperSubType, gbc_paperSubType);
                    paperSubType.setVisible(false);
                }
                {
                    //Combo box to input subtype of electronic.
                    electronicSubType.addActionListener((ActionEvent e) -> {
                        /**
                         * Set ShelfSubType to the selected item.  The combo box
                         * is populated with 'dummy class' variables.  The ActionListener
                         * uses this dummy variable value to assign a sub type.
                         */
                        if (electronicSubType.getSelectedItem() == null) {}
                        else {
                            if (electronicSubType.getSelectedItem().toString().contains(Strings.COMPUTER.toString())) {
                                shelf.setShelfSubType(ShelfSubType.COMPUTER);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.TV.toString())){
                                shelf.setShelfSubType(ShelfSubType.TV);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.LAPTOP.toString())){
                                shelf.setShelfSubType(ShelfSubType.LAPTOP);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.CRT_MONITOR.toString())){
                                shelf.setShelfSubType(ShelfSubType.CRT_MONITOR);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.LCD_MONITOR.toString())){
                                shelf.setShelfSubType(ShelfSubType.LCD_MONITOR);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.PERIPHERALS.toString())){
                                shelf.setShelfSubType(ShelfSubType.PERIPHERALS);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.PRINTER.toString())){
                                shelf.setShelfSubType(ShelfSubType.PRINTER);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.MOBILE_PHONE.toString())){
                                shelf.setShelfSubType(ShelfSubType.MOBILE_PHONE);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.DVD.toString())){
                                shelf.setShelfSubType(ShelfSubType.DVD);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.RADIO.toString())){
                                shelf.setShelfSubType(ShelfSubType.RADIO);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.APPLIANCE.toString())){
                                shelf.setShelfSubType(ShelfSubType.APPLIANCE);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.TOOL.toString())){
                                shelf.setShelfSubType(ShelfSubType.TOOL);
                            } else if (electronicSubType.getSelectedItem().toString().contains(Strings.ELECTRONICS.toString())){
                                shelf.setShelfSubType(ShelfSubType.DEF_ELEC);
                            }
                        }
                    });
                    electronicSubType.setModel(new DefaultComboBoxModel(ElectronicSubType.values()));
                    GridBagConstraints gbc_electronicSubType = new GridBagConstraints();
                    gbc_electronicSubType.fill = GridBagConstraints.HORIZONTAL;
                    gbc_electronicSubType.gridx = 0;
                    gbc_electronicSubType.gridy = 1;
                    panel.add(electronicSubType, gbc_electronicSubType);
                    electronicSubType.setVisible(false);
                }
            }
            {
                panel1.setBorder(new TitledBorder(Strings.SHELF_DIM.toString()));
                GridBagConstraints gbc_panel1 = new GridBagConstraints();
                gbc_panel1.fill = GridBagConstraints.HORIZONTAL;
                gbc_panel1.insets = new Insets(0, 0, 5, 0);
                gbc_panel1.gridx = 0;
                gbc_panel1.gridy = 1;
                contentPanel.add(panel1, gbc_panel1);
                GridBagLayout gbl_panel1 = new GridBagLayout();
                panel1.setLayout(gbl_panel1);
                {
                    Component verticalStrut = Box.createVerticalStrut(10);
                    GridBagConstraints gbc_verticalStrut = new GridBagConstraints();
                    gbc_verticalStrut.anchor = GridBagConstraints.CENTER;
                    gbc_verticalStrut.gridy = 0;
                    panel1.add(verticalStrut, gbc_verticalStrut);
                }
                {
                    //Button to input custom shelving dimensions.
                    customDim.setText(Strings.USE_CUSTOM_DIM.toString());
                    customDim.addActionListener((ActionEvent e) -> {
                        //Enable dimension input
                        shelfWidthText.setEnabled(true);
                        shelfLengthText.setEnabled(true);
                        shelfHeightText.setEnabled(true);
                        //Enable selection of default values
                        standardDim.setEnabled(true);
                        //Disable reselection of custom input
                        customDim.setEnabled(false);
                        
                    });
                    GridBagConstraints gbc_customDim = new GridBagConstraints();
                    gbc_customDim.anchor = GridBagConstraints.CENTER;
                    gbc_customDim.gridwidth = 2;
                    gbc_customDim.gridy = 1;
                    gbc_customDim.gridx = 1;
                    panel1.add(customDim, gbc_customDim);
                }
                {
                    //Button to use standard shelving dimensions
                    standardDim.setText(Strings.USE_STANDARD_DIM.toString());
                    standardDim.addActionListener((ActionEvent e) -> {
                        //Set values to standard
                        shelf.setShelfHeight(4.31);
                        shelf.setShelfLength(2.88);
                        shelf.setShelfWidth(0.92);
                        shelf.setShelfArea(0.92 * 2.88);
                        shelf.setShelfVolume(0.92 * 2.88 * 4.31);
                        //Set text input boxes to display correct value
                        shelfWidthText.setText("0.92");
                        shelfLengthText.setText("2.88");
                        shelfHeightText.setText("4.31");
                        //Disable input of custom dimensions
                        shelfWidthText.setEnabled(false);
                        shelfLengthText.setEnabled(false);
                        shelfHeightText.setEnabled(false);
                        //Re-enable button to select custom input
                        customDim.setEnabled(true);
                        //Disable reselection of standard values
                        standardDim.setEnabled(false);
                    });
                    GridBagConstraints gbc_standardDim = new GridBagConstraints();
                    gbc_standardDim.anchor = GridBagConstraints.CENTER;
                    gbc_standardDim.gridwidth = 2;
                    gbc_standardDim.gridy = 1;
                    gbc_standardDim.gridx = 3;
                    panel1.add(standardDim, gbc_standardDim);
                    standardDim.setEnabled(false);
                }
                {
                    //Label for shelf width.
                    JLabel shelfWidthLabel = new JLabel(Strings.WIDTH.toString() + ":");
                    GridBagConstraints gbc_shelfWidthLabel = new GridBagConstraints();
                    gbc_shelfWidthLabel.anchor = GridBagConstraints.CENTER;
                    gbc_shelfWidthLabel.insets = new Insets(0, 0, 0, 0);
                    gbc_shelfWidthLabel.gridy = 2;
                    gbc_shelfWidthLabel.gridx = 1;
                    panel1.add(shelfWidthLabel, gbc_shelfWidthLabel);
                }
                {
                    //Label for shelf length.
                    JLabel shelfLengthLabel = new JLabel(Strings.LENGTH.toString() + ":");
                    GridBagConstraints gbc_shelfLengthLabel = new GridBagConstraints();
                    gbc_shelfLengthLabel.anchor = GridBagConstraints.CENTER;
                    gbc_shelfLengthLabel.insets = new Insets(0, 0, 0, 0);
                    gbc_shelfLengthLabel.gridy = 3;
                    gbc_shelfLengthLabel.gridx = 1;
                    panel1.add(shelfLengthLabel, gbc_shelfLengthLabel);
                }
                {
                    //Label for shelf height.
                    JLabel shelfHeightLabel = new JLabel(Strings.HEIGHT.toString() + ":");
                    GridBagConstraints gbc_shelfHeightLabel = new GridBagConstraints();
                    gbc_shelfHeightLabel.anchor = GridBagConstraints.CENTER;
                    gbc_shelfHeightLabel.insets = new Insets(0, 0, 0, 0);
                    gbc_shelfHeightLabel.gridy = 4;
                    gbc_shelfHeightLabel.gridx = 1;
                    panel1.add(shelfHeightLabel, gbc_shelfHeightLabel);
                }
                {
                    //Text input field for width of shelf.
                    shelfWidthText.setHorizontalAlignment(SwingConstants.CENTER);                                       
                    shelfWidthText.addFocusListener(new FocusListener() {
                        /*
                         * The FocusListener selects all text in the field when
                         * focus is gained.  When focus is lost, the listener
                         * sets the value of width to text entered into the field,
                         * so long as the text input a number double.  Otherwise,
                         * the listener shows an error dialog and resets the value.
                        */
                        
                        @Override
                        public void focusGained(FocusEvent e) {
                            shelfWidthText.selectAll();
                        }

                        @Override
                        public void focusLost(FocusEvent e) {
                            String value;
                            if (shelfWidthText.getText().matches(doubleregex)) {
                                value = shelfWidthText.getText();
                                shelf.setShelfWidth(Double.parseDouble(value)); 
                                shelf.setShelfArea(Double.parseDouble(value) * 
                                        Double.parseDouble(shelfLengthText.getText()));
                                shelf.setShelfVolume(Double.parseDouble(value) * 
                                       Double.parseDouble(shelfLengthText.getText())
                                        * Double.parseDouble(shelfHeightText.getText()));
                            } else if (shelfWidthText.getText().matches(intregex)) {
                                value = shelfWidthText.getText() + ".0";
                                shelfWidthText.setText(value);
                                shelf.setShelfWidth(Double.parseDouble(value)); 
                                shelf.setShelfArea(Double.parseDouble(value) * 
                                        Double.parseDouble(shelfLengthText.getText()));
                                shelf.setShelfVolume(Double.parseDouble(value) * 
                                       Double.parseDouble(shelfLengthText.getText())
                                        * Double.parseDouble(shelfHeightText.getText()));
                            } else {
                                shelf.setShelfWidth(0.92);
                                shelfWidthText.setText("0.92");
                                shelf.setShelfArea(0.92 * Double.parseDouble(shelfLengthText.getText()));
                                shelf.setShelfVolume(0.92 * Double.parseDouble(shelfLengthText.getText())
                                    * Double.parseDouble(shelfHeightText.getText()));
                                JOptionPane.showMessageDialog(rootPane, Strings.ERROR_BASE);
                                
                            }
                        }
                    });
                    
                    GridBagConstraints gbc_shelfWidthText = new GridBagConstraints();
                    gbc_shelfWidthText.anchor = GridBagConstraints.CENTER;
                    gbc_shelfWidthText.fill = GridBagConstraints.HORIZONTAL;
                    gbc_shelfWidthText.gridy = 2;
                    gbc_shelfWidthText.gridx = 2;
                    panel1.add(shelfWidthText, gbc_shelfWidthText);
                    shelfWidthText.setColumns(10);
                    shelfWidthText.setEnabled(false);
                }
                {
                    //Text input field for length of shelf.
                    shelfLengthText.setHorizontalAlignment(SwingConstants.CENTER);
                    shelfLengthText.addFocusListener(new FocusListener() {
                        @Override
                        public void focusGained(FocusEvent e) {
                            shelfLengthText.selectAll();
                        }
                        
                        @Override
                        public void focusLost(FocusEvent e) {
                            String value;
                            if (shelfLengthText.getText().matches(doubleregex)) {
                                value = shelfLengthText.getText();
                                shelf.setShelfLength(Double.parseDouble(value));
                                shelf.setShelfArea(Double.parseDouble(shelfWidthText.getText())
                                    * Double.parseDouble(shelfLengthText.getText()));
                                shelf.setShelfVolume(Double.parseDouble(shelfWidthText.getText()) * 
                                    Double.parseDouble(shelfLengthText.getText()) * 
                                    Double.parseDouble(shelfHeightText.getText()));
                            } else if (shelfLengthText.getText().matches(intregex)) {
                                value = shelfLengthText.getText() + ".0";
                                shelfLengthText.setText(value);
                                shelf.setShelfLength(Double.parseDouble(value));
                                shelf.setShelfArea(Double.parseDouble(shelfWidthText.getText())
                                    * Double.parseDouble(value));
                                shelf.setShelfVolume(Double.parseDouble(shelfWidthText.getText()) * 
                                    Double.parseDouble(value) * 
                                    Double.parseDouble(shelfHeightText.getText()));
                            } else {
                                shelf.setShelfLength(2.88);
                                shelf.setShelfArea(2.88 * Double.parseDouble(shelfWidthText.getText()));
                                shelf.setShelfVolume(2.88 * Double.parseDouble(shelfWidthText.getText()) * 
                                        Double.parseDouble(shelfHeightText.getText()));
                                shelfLengthText.setText("2.88");
                                JOptionPane.showMessageDialog(rootPane, Strings.ERROR_BASE);  
                            }
                        }
                    });
                    GridBagConstraints gbc_shelfLengthText = new GridBagConstraints();
                    gbc_shelfLengthText.anchor = GridBagConstraints.CENTER;
                    gbc_shelfLengthText.fill = GridBagConstraints.HORIZONTAL;
                    gbc_shelfLengthText.gridy = 3;
                    gbc_shelfLengthText.gridx = 2;
                    panel1.add(shelfLengthText, gbc_shelfLengthText);
                    shelfLengthText.setColumns(10);
                    shelfLengthText.setEnabled(false);
                }
                {
                    //Text input field for height of shelf.
                    shelfHeightText.setHorizontalAlignment(SwingConstants.CENTER);
                    shelfHeightText.addFocusListener(new FocusListener() {
                        @Override
                        public void focusGained(FocusEvent e) {
                            shelfHeightText.selectAll();
                        }
                        
                        @Override
                        public void focusLost(FocusEvent e) {
                            String value;
                            if (shelfHeightText.getText().matches(doubleregex)) {
                                value = shelfHeightText.getText();
                                shelf.setShelfHeight(Double.parseDouble(value));                                                        
                                shelf.setShelfVolume(Double.parseDouble(value)
                                    * Double.parseDouble(shelfLengthText.getText()) * 
                                    Double.parseDouble(shelfWidthText.getText()));
                            } else if (shelfHeightText.getText().matches(intregex)) {
                                value = shelfHeightText.getText() + ".0";
                                shelfHeightText.setText(value);
                                shelf.setShelfHeight(Double.parseDouble(value));                                                        
                                shelf.setShelfVolume(Double.parseDouble(value)
                                    * Double.parseDouble(shelfLengthText.getText()) * 
                                    Double.parseDouble(shelfWidthText.getText()));
                            } else {
                                shelf.setShelfHeight(4.31);
                                shelf.setShelfVolume(4.31 * Double.parseDouble(shelfLengthText.getText()) *
                                        Double.parseDouble(shelfWidthText.getText()));
                                shelfHeightText.setText("4.31");
                                JOptionPane.showMessageDialog(rootPane, Strings.ERROR_BASE);
                            }
                        }
                    });
                    GridBagConstraints gbc_shelfHeightText = new GridBagConstraints();
                    gbc_shelfHeightText.anchor = GridBagConstraints.CENTER;
                    gbc_shelfHeightText.fill = GridBagConstraints.HORIZONTAL;
                    gbc_shelfHeightText.gridy = 4;
                    gbc_shelfHeightText.gridx = 2;
                    panel1.add(shelfHeightText, gbc_shelfHeightText);
                    shelfHeightText.setColumns(10);
                    shelfHeightText.setEnabled(false);
                }
                {
                    //Unit label for shelf width.
                    JLabel shelfWidthUnitLabel = new JLabel(" m");
                    GridBagConstraints gbc_shelfWidthUnitLabel = new GridBagConstraints();
                    gbc_shelfWidthUnitLabel.anchor = GridBagConstraints.LINE_START;
                    gbc_shelfWidthUnitLabel.gridy = 2;
                    gbc_shelfWidthUnitLabel.gridx = 3;
                    panel1.add(shelfWidthUnitLabel, gbc_shelfWidthUnitLabel);
                }
                {
                    //Unit label for shelf length.
                    JLabel shelfLengthUnitLabel = new JLabel(" m");
                    GridBagConstraints gbc_shelfLengthUnitLabel = new GridBagConstraints();
                    gbc_shelfLengthUnitLabel.anchor = GridBagConstraints.LINE_START;
                    gbc_shelfLengthUnitLabel.gridy = 3;
                    gbc_shelfLengthUnitLabel.gridx = 3;
                    panel1.add(shelfLengthUnitLabel, gbc_shelfLengthUnitLabel);
                }
                {
                    //Unit label for shelf height.
                    JLabel shelfHeightUnitLabel = new JLabel(" m");
                    GridBagConstraints gbc_shelfHeightUnitLabel = new GridBagConstraints();
                    gbc_shelfHeightUnitLabel.anchor =  GridBagConstraints.LINE_START;
                    gbc_shelfHeightUnitLabel.gridy = 4;
                    gbc_shelfHeightUnitLabel.gridx = 3;
                    panel1.add(shelfHeightUnitLabel, gbc_shelfHeightUnitLabel);
                }
                {
                    //Help panel for shelf height
                    ImageIcon heightIcon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/shelfheight.jpg"));
                    heightHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "La altura de la unidad del estante.\nLa parte resaltada de la imagen denota la altura.",
                                Strings.HEIGHT.toString(), JOptionPane.INFORMATION_MESSAGE, heightIcon);
                    });
                    GridBagConstraints gbc_heightHelp = new GridBagConstraints();
                    gbc_heightHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_heightHelp.gridy = 4;
                    gbc_heightHelp.gridx = 4;
                    panel1.add(heightHelp, gbc_heightHelp);
                }
                {
                    //Help panel for shelf length.
                    ImageIcon lengthIcon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/shelflength.jpg"));
                    lengthHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "El longitud de la unidad del estante.\nLa parte resaltada de la imagen denota el longitud.",
                                Strings.LENGTH.toString(), JOptionPane.INFORMATION_MESSAGE, lengthIcon);
                    });
                    GridBagConstraints gbc_lengthHelp = new GridBagConstraints();
                    gbc_lengthHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_lengthHelp.gridy = 3;
                    gbc_lengthHelp.gridx = 4;
                    panel1.add(lengthHelp, gbc_lengthHelp);
                }
                {
                    //Help panel for shelf width.
                    ImageIcon widthIcon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/shelfwidth.jpg"));
                    widthHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "La anchura de la unidad del estante.\nLa parte resaltada de la imagen denota la anchura.",
                                Strings.WIDTH.toString(), JOptionPane.INFORMATION_MESSAGE, widthIcon);
                    });
                    GridBagConstraints gbc_widthHelp = new GridBagConstraints();
                    gbc_widthHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_widthHelp.gridy = 2;
                    gbc_widthHelp.gridx = 4;
                    panel1.add(widthHelp, gbc_widthHelp);
                }
                {
                    Component box = Box.createVerticalStrut(20);
                    GridBagConstraints gbc_box = new GridBagConstraints();
                    gbc_box.anchor = GridBagConstraints.CENTER;
                    gbc_box.fill = GridBagConstraints.HORIZONTAL;
                    gbc_box.gridy = 5;
                    panel1.add(box, gbc_box);
                }
                {
                    //Label for number of shelf levels
                    JLabel levelLabel = new JLabel(Strings.SHELF_LVLS.toString() + ":");
                    GridBagConstraints gbc_levelLabel = new GridBagConstraints();
                    gbc_levelLabel.anchor = GridBagConstraints.CENTER;
                    gbc_levelLabel.gridx = 1;
                    gbc_levelLabel.gridy = 6;
                    panel1.add(levelLabel, gbc_levelLabel);
                }
                {
                    //Input for number of levels on a shelf.
                    shelfLevels.setHorizontalAlignment(SwingConstants.CENTER);
                    shelfLevels.addFocusListener(new FocusListener() {
                        /**
                         * The focus listener selects all text, and enables the
                         * next input field when focus is gained.  When focus is
                         * lost, the listener sets the levels of shelving to the
                         * text input, so long as the input is a numeric integer.
                         * Otherwise, the listener displays an error message and
                         * resets the value.
                         */
                        @Override
                        public void focusGained(FocusEvent e) {
                            shelfLevels.selectAll();
                        }
                        
                        @Override
                        public void focusLost(FocusEvent e) {
                            String levels = shelfLevels.getText();
                            if (levels.matches(intregex)) {
                                shelf.setShelfLevels(Integer.parseInt(levels));
                            } else {
                                JOptionPane.showMessageDialog(rootPane, Strings.ERROR_BASE);
                                shelf.setShelfLevels(1);
                                shelfLevels.setText("1");
                            }
                            
                        }


                    });
                    GridBagConstraints gbc_shelfLevels = new GridBagConstraints();
                    gbc_shelfLevels.anchor = GridBagConstraints.CENTER;
                    gbc_shelfLevels.fill = GridBagConstraints.HORIZONTAL;
                    gbc_shelfLevels.gridy = 6;
                    gbc_shelfLevels.gridx = 2;
                    shelfLevels.setColumns(10);
                    panel1.add(shelfLevels, gbc_shelfLevels);    
                }
                {
                    //Help panel for shelf levels
                    ImageIcon shelfIcon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/shelflevel.jpg"));
                    levelHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "El número de niveles en la unidad del estante.\nLa parte resaltada de la imagen denota los niveles.\nEl ejemplo tiene 5 niveles.",
                                Strings.SHELF_LVLS.toString(), JOptionPane.INFORMATION_MESSAGE, shelfIcon);
                    });
                    GridBagConstraints gbc_levelHelp = new GridBagConstraints();
                    gbc_levelHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_levelHelp.gridx = 4;
                    gbc_levelHelp.gridy = 6;
                    panel1.add(levelHelp, gbc_levelHelp);
                }
                {
                    Component box = Box.createHorizontalStrut(0);
                    GridBagConstraints gbc_box = new GridBagConstraints();
                    gbc_box.anchor = GridBagConstraints.CENTER;
                    gbc_box.gridx = 3;
                    gbc_box.gridy = 1;
                    gbc_box.gridheight = 3;
                    panel1.add(box, gbc_box);
                }
            }
            {
                panel6.setBorder(new TitledBorder(Strings.SHELF_BASE.toString()));
                GridBagConstraints gbc_panel6 = new GridBagConstraints();
                gbc_panel6.insets = new Insets(0, 0, 5, 0);
                gbc_panel6.fill = GridBagConstraints.BOTH;
                gbc_panel6.gridx = 0;
                gbc_panel6.gridy = 2;
                contentPanel.add(panel6, gbc_panel6);
                GridBagLayout gbl_panel6 = new GridBagLayout();
                gbl_panel6.columnWidths = new int[]{0, 0};
                gbl_panel6.rowHeights = new int[]{0, 0, 0};
                gbl_panel6.columnWeights = new double[]{1.0, Double.MIN_VALUE};
                gbl_panel6.rowWeights = new double[]{1.0, 0.0, Double.MIN_VALUE};
                panel6.setLayout(gbl_panel6);
                
                {
                    //Combo box to select type of base used on shelving.
                    shelfBase.addActionListener((ActionEvent e) -> {
                        shelf.setShelfBaseType((ShelfBaseType) shelfBase.getSelectedItem());
                    });
                    shelfBase.setModel(new DefaultComboBoxModel(ShelfBaseType.values()));
                    //Set initial value to <null>
                    shelfBase.setSelectedItem(null);
                    GridBagConstraints gbc_shelfBase = new GridBagConstraints();
                    gbc_shelfBase.fill = GridBagConstraints.HORIZONTAL;
                    gbc_shelfBase.gridx = 0;
                    gbc_shelfBase.gridy = 1;
                    panel6.add(shelfBase, gbc_shelfBase);
                }
                {                  
                    //Help dialog for shelf base.
                    ImageIcon baseIcon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/nobase.jpg"));
                    baseHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "El material que compone la base.\nEl ejemplo muestra un estante sin la base.\n"
                                        + "Si la unidad del estante es un refrigerador, elija la opción 'Refrigerador'.",
                                Strings.SHELF_BASE.toString(), JOptionPane.INFORMATION_MESSAGE, baseIcon);
                    });
                    GridBagConstraints gbc_baseHelp = new GridBagConstraints();
                    gbc_baseHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_baseHelp.gridx = 1;
                    gbc_baseHelp.gridy = 1;
                    panel6.add(baseHelp, gbc_baseHelp);
                }
            }
            
            Component horizontalstrut = Box.createHorizontalStrut(20);
            GridBagConstraints gbc_horizontalStrut = new GridBagConstraints();
            gbc_horizontalStrut.anchor = GridBagConstraints.CENTER;
            gbc_horizontalStrut.gridx = 1;
            contentPanel.add(horizontalstrut, gbc_horizontalStrut);
            
            {
                panel4.setBorder(new TitledBorder(Strings.SHELF_QUANTITY.toString()));
                GridBagConstraints gbc_panel4 = new GridBagConstraints();
                gbc_panel4.insets = new Insets(0, 0, 5, 0);
                gbc_panel4.fill = GridBagConstraints.BOTH;
                gbc_panel4.gridx = 2;
                gbc_panel4.gridy = 0;
                contentPanel.add(panel4, gbc_panel4);
                GridBagLayout gbl_panel4 = new GridBagLayout();
                gbl_panel4.columnWidths = new int[]{0, 0};
                gbl_panel4.rowHeights = new int[]{0, 0, 0};
                gbl_panel4.columnWeights = new double[]{1.0, Double.MIN_VALUE};
                gbl_panel4.rowWeights = new double[]{1.0, 0.0, Double.MIN_VALUE};
                panel4.setLayout(gbl_panel4);
            
                {
                    Component verticalStrut = Box.createVerticalStrut(10);
                    GridBagConstraints gbc_verticalStrut = new GridBagConstraints();
                    gbc_verticalStrut.anchor = GridBagConstraints.CENTER;
                    gbc_verticalStrut.insets = new Insets(0, 0, 0, 0);
                    gbc_verticalStrut.gridx = 0;
                    gbc_verticalStrut.gridy = 0;
                    panel4.add(verticalStrut, gbc_verticalStrut);
                }
                {
                    //Quantity field for number of rooms to be added.
                    shelfQuantity.addFocusListener(new FocusListener() {
                        /**
                         * The FocusListener highlights all text in the field and
                         * enables the 'Add' button when focus is gained.  When
                         * focus is lost, the listener sets the quantity of shelving
                         * units to the text entered in the field, so long as the
                         * text is a numeric integer.  Otherwise, an error message
                         * is displayed, and the value is reset.
                         */
                        @Override
                        public void focusGained(FocusEvent e) {
                            shelfQuantity.selectAll();
                        }
                        @Override
                        public void focusLost(FocusEvent e) {
                            String quantity = shelfQuantity.getText();
                            if (quantity.matches(intregex)) {
                                shelf.setShelfQuantity(Integer.parseInt(shelfQuantity.getText()));
                            } else {
                                JOptionPane.showMessageDialog(rootPane, Strings.ERROR_BASE);
                                shelfQuantity.setText("1");
                                shelf.setShelfQuantity(1);
                            }
                        }
                    });
                    GridBagConstraints gbc_shelfQuantity = new GridBagConstraints();
                    gbc_shelfQuantity.anchor = GridBagConstraints.CENTER;
                    gbc_shelfQuantity.fill = GridBagConstraints.HORIZONTAL;
                    gbc_shelfQuantity.gridx = 0;
                    gbc_shelfQuantity.gridy = 1;
                    shelfQuantity.setColumns(10);
                    shelfQuantity.setHorizontalAlignment(SwingConstants.RIGHT);
                    panel4.add(shelfQuantity, gbc_shelfQuantity);
                }
                {                   
                    //Help dialog for shelf quantity.
                    ImageIcon shpi = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
                    quantityHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "El número de estantes idénticas en la bodega.",
                                Strings.QUANTITY.toString(), JOptionPane.INFORMATION_MESSAGE, shpi);
                    });
                    GridBagConstraints gbc_quantityHelp = new GridBagConstraints();
                    gbc_quantityHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_quantityHelp.gridx = 1;
                    gbc_quantityHelp.gridy = 1;
                    panel4.add(quantityHelp, gbc_quantityHelp);
                }
                {
                    Component verticalStrut = Box.createVerticalStrut(10);
                    GridBagConstraints gbc_verticalStrut = new GridBagConstraints();
                    gbc_verticalStrut.anchor = GridBagConstraints.CENTER;
                    gbc_verticalStrut.gridx = 0;
                    gbc_verticalStrut.gridy = 2;
                    panel4.add(verticalStrut, gbc_verticalStrut);
                }
            }
            {
                panel3.setBorder(new TitledBorder(new EtchedBorder(
                        EtchedBorder.LOWERED, null, null), Strings.USAGE
                        .toString(), TitledBorder.LEADING, TitledBorder.TOP, null,
                        null));
                GridBagConstraints gbc_panel3 = new GridBagConstraints();
                gbc_panel3.fill = GridBagConstraints.BOTH;
                gbc_panel3.gridx = 2;
                gbc_panel3.gridy = 1;
                contentPanel.add(panel3, gbc_panel3);
                GridBagLayout gbl_panel3 = new GridBagLayout();
                panel3.setLayout(gbl_panel3);
                
                {
                    Component box = Box.createHorizontalStrut(113);
                    GridBagConstraints gbc_box = new GridBagConstraints();
                    gbc_box.anchor = GridBagConstraints.LINE_START;
                    gbc_box.fill = GridBagConstraints.HORIZONTAL;
                    gbc_box.gridy = 0;
                    gbc_box.gridx = 0;
                    gbc_box.gridheight = 2;
                    panel3.add(box, gbc_box);
                }
                {
                    //Text label for the usage slider.
                    JLabel lblNewLabel = new JLabel(
                            Strings.PERCENT_USAGE.toString() + ":");
                    GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
                    gbc_lblNewLabel.anchor = GridBagConstraints.CENTER;
                    gbc_lblNewLabel.gridy = 0;
                    gbc_lblNewLabel.gridx = 1;
                    panel3.add(lblNewLabel, gbc_lblNewLabel);
                }
                {
                    //Display for percent.
                    GridBagConstraints gbc_usageLabel = new GridBagConstraints();
                    gbc_usageLabel.anchor = GridBagConstraints.CENTER;
                    gbc_usageLabel.gridy = 0;
                    gbc_usageLabel.gridx = 2;
                    panel3.add(usageLabel, gbc_usageLabel);
                }
                {
                    //Slider to input percentage of shelf area used.
                    usageSlider.addChangeListener((ChangeEvent arg0) -> {
                        usageLabel.setText(String.valueOf(usageSlider
                                .getValue()) + "%");
                        shelf.setAreaUsage(usageSlider.getValue());
                    });
                    usageSlider.setValue(100);
                    usageSlider.setMajorTickSpacing(10);
                    usageSlider.setPaintTicks(true);
                    usageSlider.setPaintLabels(true);
                    GridBagConstraints gbc_usageSlider = new GridBagConstraints();
                    gbc_usageSlider.anchor = GridBagConstraints.CENTER;
                    gbc_usageSlider.gridy = 1;
                    gbc_usageSlider.gridx = 1;
                    gbc_usageSlider.gridwidth = 2;
                    gbc_usageSlider.fill = GridBagConstraints.HORIZONTAL;
                    panel3.add(usageSlider, gbc_usageSlider);
                }
                {
                    Component box = Box.createHorizontalStrut(43);
                    GridBagConstraints gbc_box = new GridBagConstraints();
                    gbc_box.anchor = GridBagConstraints.CENTER;
                    gbc_box.gridy = 0;
                    gbc_box.gridheight = 2;
                    gbc_box.gridx = 3;
                    panel3.add(box, gbc_box);
                }
                {
                    //Help dialog for usage percentage.
                    ImageIcon usageIcon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/areausage.jpg"));
                    usageHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "El porcentaje de la área utilizada de la unidad del estante.\nEl ejemplo se ve el 60% del área usada.",
                                Strings.USAGE.toString(), JOptionPane.INFORMATION_MESSAGE, usageIcon);
                    });
                    GridBagConstraints gbc_usageHelp = new GridBagConstraints();
                    gbc_usageHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_usageHelp.gridheight = 2;
                    gbc_usageHelp.gridy = 1;
                    gbc_usageHelp.gridx = 4;
                    panel3.add(usageHelp, gbc_usageHelp);
                }
                {
                    Component box = Box.createVerticalStrut(20);
                    GridBagConstraints gbc_box = new GridBagConstraints();
                    gbc_box.anchor = GridBagConstraints.LINE_START;
                    gbc_box.fill = GridBagConstraints.HORIZONTAL;
                    gbc_box.gridy = 3;
                    gbc_box.gridx = 0;
                    gbc_box.gridheight = 1;
                    panel3.add(box, gbc_box);
                    
                }
                {
                    JLabel label = new JLabel(Strings.PERCENT_HEIGHT.toString() + ":");
                    GridBagConstraints gbc_label = new GridBagConstraints();
                    gbc_label.anchor = GridBagConstraints.CENTER;
                    gbc_label.gridy = 4;
                    gbc_label.gridx = 1;
                    panel3.add(label, gbc_label);
                }
                {
                    //Slider to input percentage of shelf height used.
                    heightusageslider.addChangeListener((ChangeEvent arg0) -> {
                        heightUsageLabel.setText(String.valueOf(heightusageslider
                                .getValue()) + "%");
                        shelf.setHeightUsage(heightusageslider.getValue());
                    });
                    heightusageslider.setValue(100);
                    heightusageslider.setMajorTickSpacing(10);
                    heightusageslider.setPaintTicks(true);
                    heightusageslider.setPaintLabels(true);
                    GridBagConstraints gbc_heightusageslider = new GridBagConstraints();
                    gbc_heightusageslider.anchor = GridBagConstraints.CENTER;
                    gbc_heightusageslider.gridy = 5;
                    gbc_heightusageslider.gridx = 1;
                    gbc_heightusageslider.gridwidth = 2;
                    gbc_heightusageslider.fill = GridBagConstraints.HORIZONTAL;
                    panel3.add(heightusageslider, gbc_heightusageslider);
                }
                {

                    GridBagConstraints gbc_heightUsageLabel = new GridBagConstraints();
                    gbc_heightUsageLabel.anchor = GridBagConstraints.CENTER;
                    gbc_heightUsageLabel.gridy = 4;
                    gbc_heightUsageLabel.gridx = 2;
                    panel3.add(heightUsageLabel, gbc_heightUsageLabel);
                }
                {
                    //Help dialog for usage percentage.
                    ImageIcon usageIcon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/heightusage.jpg"));
                    heightUsageHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "El porcentaje de la altura utilizada del estante.\nEn el ejemplo se ve el 60% de la altura usada.",
                                Strings.USAGE.toString(), JOptionPane.INFORMATION_MESSAGE, usageIcon);
                    });
                    GridBagConstraints gbc_heightUsageHelp = new GridBagConstraints();
                    gbc_heightUsageHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_heightUsageHelp.gridheight = 2;
                    gbc_heightUsageHelp.gridy = 4;
                    gbc_heightUsageHelp.gridx = 4;
                    panel3.add(heightUsageHelp, gbc_heightUsageHelp);
                }
            }
            {
                panel5.setBorder(new TitledBorder(new EtchedBorder(
                    EtchedBorder.LOWERED, null, null), Strings.DAMAGE
                    .toString(), TitledBorder.LEADING, TitledBorder.TOP, null,
                    null));
                GridBagConstraints gbc_panel5 = new GridBagConstraints();
                gbc_panel5.fill = GridBagConstraints.BOTH;
                gbc_panel5.gridx = 2;
                gbc_panel5.gridy = 2;
                gbc_panel5.gridheight = 2;
                contentPanel.add(panel5, gbc_panel5);
                GridBagLayout gbl_panel5 = new GridBagLayout();
                panel5.setLayout(gbl_panel5);

                {
                    Component box = Box.createHorizontalStrut(113);
                    GridBagConstraints gbc_box = new GridBagConstraints();
                    gbc_box.anchor = GridBagConstraints.LINE_START;
                    gbc_box.fill = GridBagConstraints.HORIZONTAL;
                    gbc_box.gridy = 0;
                    gbc_box.gridx = 0;
                    gbc_box.gridheight = 2;
                    panel5.add(box, gbc_box);
                }
                {
                    //Text label for the damage slider.
                    JLabel lblNewLabel = new JLabel(
                            Strings.PERCENT_DAMAGE.toString() + ":");
                    GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
                    gbc_lblNewLabel.insets = new Insets(0, 0, 5, 5);
                    gbc_lblNewLabel.anchor = GridBagConstraints.CENTER;
                    gbc_lblNewLabel.gridx = 1;
                    gbc_lblNewLabel.gridy = 0;
                    panel5.add(lblNewLabel, gbc_lblNewLabel);
                }
                {

                    GridBagConstraints gbc_damageLabel = new GridBagConstraints();
                    gbc_damageLabel.anchor = GridBagConstraints.CENTER;
                    gbc_damageLabel.insets = new Insets(0, 0, 5, 0);
                    gbc_damageLabel.gridx = 2;
                    gbc_damageLabel.gridy = 0;
                    panel5.add(damageLabel, gbc_damageLabel);
                }
                {
                    //Slider to input percentage of room damaged.
                    damageSlider.addChangeListener((ChangeEvent arg0) -> {
                        damageLabel.setText(String.valueOf(damageSlider
                                .getValue()) + "%");
                        shelf.setDamage(damageSlider.getValue());
                    });
                    damageSlider.setValue(100);
                    damageSlider.setMajorTickSpacing(10);
                    damageSlider.setPaintTicks(true);
                    damageSlider.setPaintLabels(true);
                    GridBagConstraints gbc_damageSlider = new GridBagConstraints();
                    gbc_damageSlider.anchor = GridBagConstraints.CENTER;
                    gbc_damageSlider.gridwidth = 2;
                    gbc_damageSlider.gridx = 1;
                    gbc_damageSlider.gridy = 1;
                    panel5.add(damageSlider, gbc_damageSlider);
                }
                {
                    //Help dialog for damage percentage.
                    ImageIcon damageIcon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/damageIcon.jpg"));
                    damageHelp.addActionListener((ActionEvent e) -> {
                        JOptionPane.showMessageDialog(null,
                                "El porcentaje de la unidad del enstante que fue dañado.\nEl ejempro se ve un daño de 50%.",
                                Strings.DAMAGE.toString(), JOptionPane.INFORMATION_MESSAGE, damageIcon);
                    });
                    GridBagConstraints gbc_damageHelp = new GridBagConstraints();
                    gbc_damageHelp.anchor = GridBagConstraints.LINE_END;
                    gbc_damageHelp.gridheight = 2;
                    gbc_damageHelp.gridx = 4;
                    panel5.add(damageHelp, gbc_damageHelp);
                }
                {
                    Component box = Box.createHorizontalStrut(48);
                    GridBagConstraints gbc_box = new GridBagConstraints();
                    gbc_box.anchor = GridBagConstraints.CENTER;
                    gbc_box.gridy = 0;
                    gbc_box.gridheight = 2;
                    gbc_box.gridx = 3;
                    panel5.add(box, gbc_box);
                }
            }
            {
                JPanel buttonPane = new JPanel();
                buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
                getContentPane().add(buttonPane, BorderLayout.SOUTH);
                
                {
                    //Cancel button to discard current form
                    JButton cancelButton = new JButton(Strings.CANCEL.toString());
                    cancelButton.addActionListener((ActionEvent e) -> {
                        setVisible(false);
                        dispose();
                    });
                    buttonPane.add(cancelButton);
                }
                {
                    /**
                     * Adds the current form as a room to the structure.
                     * Prompts user prior to adding form.
                     * When adding form, validates input.
                     */
                    addButton.addActionListener((ActionEvent e) -> {
                        ImageIcon imageicon = new ImageIcon(ShelfDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
                        if (shelf.getShelfType() == null) {
                            JOptionPane.showMessageDialog(null, Strings.PLEASE_CHOOSE.toString() + " " + Strings.SHELF_TYPE, Strings.ERROR.toString(), JOptionPane.INFORMATION_MESSAGE, imageicon);
                            setVisible(true);
                        } else if (shelf.getShelfSubType() == null) {
                            JOptionPane.showMessageDialog(null, Strings.PLEASE_CHOOSE.toString() +  " " + Strings.SHELF_SUB, Strings.ERROR.toString(), JOptionPane.INFORMATION_MESSAGE, imageicon);
                            setVisible(true);
                        } else if (shelf.getShelfBaseType() ==  null) {
                            JOptionPane.showMessageDialog(null, Strings.PLEASE_CHOOSE.toString() +  " " + Strings.SHELF_BASE, Strings.ERROR.toString(), JOptionPane.INFORMATION_MESSAGE, imageicon);
                            setVisible(true);
                        } else if ((ware.getTotalRoomArea() + (shelf.getShelfArea() * shelf.getShelfQuantity())) > Warehouse.getAreaTotal()) {
                                    JOptionPane.showMessageDialog(this, Strings.ROOM_AREA_BIGGER);
                                    setVisible(true);
                        } else {
                            int response = JOptionPane.showConfirmDialog(null, Strings.CONFIRM_DATA, Strings.CONFIRM.toString(), JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE, imageicon);
                            if (response == JOptionPane.CANCEL_OPTION) {
                                setVisible(true);
                            } else if (response == JOptionPane.OK_OPTION) {
                                    ware.addTotalRoomArea(shelf.getShelfArea() * shelf.getShelfQuantity());
                                    getParentForm().addShelf(shelf);
                                    dispose();
                                    setVisible(false);
                            }
                        }
                    });
                buttonPane.add(addButton); 
                rootPane.setDefaultButton(addButton);
                
                }
                
                setVisible(true);
            }
            
        }
        
    }   

    public AdvancedMainForm getParentForm() {
        return parent;
    }

}