/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.visual.structure;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.Window.Type;

import javax.swing.JComboBox;
import java.awt.GridBagConstraints;
import javax.swing.DefaultComboBoxModel;

import sa.shpi.advanced.model.EmissionsItem;
import sa.shpi.advanced.model.Strings;
import sa.shpi.advanced.model.structure.Section;
import sa.shpi.advanced.model.structure.SectionCategory;
import sa.shpi.advanced.model.structure.StructureUsedMaterials;
import sa.shpi.advanced.visual.AdvancedMainForm;
import sa.shpi.advanced.visual.EmissionsItemRequesterForm;

import java.awt.Insets;
import javax.swing.border.TitledBorder;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import java.awt.Font;
import java.awt.Component;
import javax.swing.Box;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Dialog.ModalityType;
import java.util.ArrayList;
import java.util.EnumSet;
import javax.swing.ImageIcon;
import javax.swing.JList;
import javax.swing.JScrollPane;
import sa.shpi.advanced.model.structure.MaterialList;

/**
 * MaterialDetailForm receives a material and allows the user to adjust the
 * percent of the selected structural location that is comprised of that
 * material
 *
 * @author Kyle
 *
 */
public class MaterialDetailForm extends EmissionsItemRequesterForm {

    private static final long serialVersionUID = 1L;

    AdvancedMainForm parent;

    private JTextField materialPane;
    private JComboBox<SectionCategory> sectionComboBox;

    private EmissionsItem material;
    private JList materialList;

    private JLabel densityLabel;
    private JLabel emissionsFactorLabel;
    
    public MaterialDetailForm(AdvancedMainForm parent) {
        super(parent.getFrame());
        this.parent = parent;
        this.material = new EmissionsItem(Strings.NONE.toString(), 0.00, 0.00);
        initialize();
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        
        //Set initial value for section
        parent.getTool().getWarehouse().setWareSectionCategory(SectionCategory.EXTERNAL_WALLS);
        
        setTitle(Strings.MATERIAL.toString());
        setType(Type.UTILITY);
        setModalityType(ModalityType.APPLICATION_MODAL);
        setResizable(false);
        setBounds(100, 100, 288, 300);

        JPanel panel = new JPanel();
        getContentPane().add(panel, BorderLayout.CENTER);
        GridBagLayout gbl_panel = new GridBagLayout();
        panel.setLayout(gbl_panel);

        densityLabel = new JLabel(String.format("%.2f", material.getMass())
                + " kg/m^2");
        emissionsFactorLabel = new JLabel(Double.toString(material
                .getEmissionsFactor()));

        JPanel panel_1 = new JPanel();
        panel_1.setBorder(new TitledBorder(null, Strings.MATERIAL_LOCATION
                .toString(), TitledBorder.LEADING, TitledBorder.TOP, null, null));
        GridBagConstraints gbc_panel_1 = new GridBagConstraints();
        gbc_panel_1.insets = new Insets(0, 0, 5, 0);
        gbc_panel_1.fill = GridBagConstraints.BOTH;
        gbc_panel_1.gridx = 0;
        gbc_panel_1.gridy = 0;
        panel.add(panel_1, gbc_panel_1);
        GridBagLayout gbl_panel_1 = new GridBagLayout();
        gbl_panel_1.columnWidths = new int[]{94, 0};
        gbl_panel_1.rowHeights = new int[]{22, 0};
        gbl_panel_1.columnWeights = new double[]{1.0, Double.MIN_VALUE};
        gbl_panel_1.rowWeights = new double[]{0.0, Double.MIN_VALUE};
        panel_1.setLayout(gbl_panel_1);

        {
            sectionComboBox = new JComboBox<SectionCategory>();
            sectionComboBox.addActionListener((ActionEvent e) -> {
                parent.getTool().getWarehouse().setWareSectionCategory((SectionCategory) sectionComboBox.getSelectedItem());
            });
            sectionComboBox.setModel(new DefaultComboBoxModel(
                    SectionCategory.values()));
            GridBagConstraints gbc_sectionComboBox = new GridBagConstraints();
            gbc_sectionComboBox.fill = GridBagConstraints.HORIZONTAL;
            gbc_sectionComboBox.anchor = GridBagConstraints.NORTH;
            gbc_sectionComboBox.gridx = 0;
            gbc_sectionComboBox.gridy = 0;
            panel_1.add(sectionComboBox, gbc_sectionComboBox);
        }
        {
            ImageIcon sectionIcon = new ImageIcon(MaterialDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
            JButton sectionHelp = new JButton("?");
            sectionHelp.addActionListener((ActionEvent e) -> {
                JOptionPane.showMessageDialog(null, "Selección para la parte especifica de la estructura.",
                        Strings.MATERIAL_LOCATION.toString(),
                        JOptionPane.INFORMATION_MESSAGE, sectionIcon);
            });
            GridBagConstraints gbc_sectionHelp = new GridBagConstraints();
            gbc_sectionHelp.anchor = GridBagConstraints.CENTER;
            gbc_sectionHelp.gridx = 1;
            gbc_sectionHelp.gridy = 0;
            panel_1.add(sectionHelp, gbc_sectionHelp);
        }

        JPanel panel_2 = new JPanel();
        panel_2.setBorder(new TitledBorder(null, Strings.MATERIAL.toString(),
                TitledBorder.LEADING, TitledBorder.TOP, null, null));
        GridBagConstraints gbc_panel_2 = new GridBagConstraints();
        gbc_panel_2.insets = new Insets(0, 0, 5, 0);
        gbc_panel_2.fill = GridBagConstraints.BOTH;
        gbc_panel_2.gridx = 0;
        gbc_panel_2.gridy = 1;
        panel.add(panel_2, gbc_panel_2);
        GridBagLayout gbl_panel_2 = new GridBagLayout();
        gbl_panel_2.columnWidths = new int[]{0, 0, 0};
        gbl_panel_2.rowHeights = new int[]{0, 0, 0, 0};
        gbl_panel_2.columnWeights = new double[]{1.0, 0.0, Double.MIN_VALUE};
        gbl_panel_2.rowWeights = new double[]{0.0, 0.0, 0.0, Double.MIN_VALUE};
        panel_2.setLayout(gbl_panel_2);

        Component verticalStrut_1 = Box.createVerticalStrut(10);
        GridBagConstraints gbc_verticalStrut_1 = new GridBagConstraints();
        gbc_verticalStrut_1.insets = new Insets(0, 0, 5, 5);
        gbc_verticalStrut_1.gridx = 0;
        gbc_verticalStrut_1.gridy = 0;
        panel_2.add(verticalStrut_1, gbc_verticalStrut_1);

        {
            materialPane = new JTextField();
            materialPane.setEditable(false);
            materialPane.setText(material.getName());
            GridBagConstraints gbc_materialList = new GridBagConstraints();
            gbc_materialList.insets = new Insets(0, 0, 5, 0);
            gbc_materialList.fill = GridBagConstraints.BOTH;
            gbc_materialList.gridx = 0;
            gbc_materialList.gridy = 1;
            panel_2.add(materialPane, gbc_materialList);
        }

        JButton materialSelectionButton = new JButton(Strings.CHOOSE.toString()
                + "...");
        materialSelectionButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                parent.getTool().getWarehouse().initializelibrary();
                MaterialChooseForm matForm = new MaterialChooseForm(
                        MaterialDetailForm.this, parent.getTool().getWarehouse()
                        .getMaterialsLibrary());
            }
        });
        GridBagConstraints gbc_materialSelectionButton = new GridBagConstraints();
        gbc_materialSelectionButton.insets = new Insets(0, 0, 5, 0);
        gbc_materialSelectionButton.gridx = 1;
        gbc_materialSelectionButton.gridy = 1;
        panel_2.add(materialSelectionButton, gbc_materialSelectionButton);

        Component verticalStrut_7 = Box.createVerticalStrut(10);
        GridBagConstraints gbc_verticalStrut_7 = new GridBagConstraints();
        gbc_verticalStrut_7.insets = new Insets(0, 0, 0, 5);
        gbc_verticalStrut_7.gridx = 0;
        gbc_verticalStrut_7.gridy = 2;
        panel_2.add(verticalStrut_7, gbc_verticalStrut_7);

        JPanel panel_3 = new JPanel();
        panel_3.setBorder(new TitledBorder(null, Strings.DETAILS.toString(),
                TitledBorder.LEADING, TitledBorder.TOP, null, null));
        GridBagConstraints gbc_panel_3 = new GridBagConstraints();
        gbc_panel_3.anchor = GridBagConstraints.CENTER;
        gbc_panel_3.fill = GridBagConstraints.HORIZONTAL;
        gbc_panel_3.gridx = 0;
        gbc_panel_3.gridy = 2;
        panel.add(panel_3, gbc_panel_3);
        GridBagLayout gbl_panel_3 = new GridBagLayout();
        panel_3.setLayout(gbl_panel_3);

        JLabel lblEmissionsFactor = new JLabel(
                Strings.EMISSIONS_FACTOR.toString() + ":");
        lblEmissionsFactor.setFont(new Font("Tahoma", Font.BOLD, 11));
        GridBagConstraints gbc_lblEmissionsFactor = new GridBagConstraints();
        gbc_lblEmissionsFactor.anchor = GridBagConstraints.LINE_END;
        gbc_lblEmissionsFactor.gridx = 0;
        gbc_lblEmissionsFactor.gridy = 0;
        panel_3.add(lblEmissionsFactor, gbc_lblEmissionsFactor);

        GridBagConstraints gbc_emissionsFactorLabel = new GridBagConstraints();
        gbc_emissionsFactorLabel.anchor = GridBagConstraints.LINE_START;
        gbc_emissionsFactorLabel.gridx = 1;
        gbc_emissionsFactorLabel.gridy = 0;
        panel_3.add(emissionsFactorLabel, gbc_emissionsFactorLabel);

        JLabel lblNewLabel_1 = new JLabel(Strings.MASS.toString() + ":");
        lblNewLabel_1.setFont(new Font("Tahoma", Font.BOLD, 11));
        GridBagConstraints gbc_lblNewLabel_1 = new GridBagConstraints();
        gbc_lblNewLabel_1.anchor = GridBagConstraints.LINE_END;
        gbc_lblNewLabel_1.gridx = 0;
        gbc_lblNewLabel_1.gridy = 1;
        panel_3.add(lblNewLabel_1, gbc_lblNewLabel_1);

        densityLabel = new JLabel(String.format("%.2f", material.getMass())
                + " kg/m^2");
        GridBagConstraints gbc_densityLabel = new GridBagConstraints();
        gbc_densityLabel.anchor = GridBagConstraints.NORTHWEST;
        gbc_densityLabel.gridx = 1;
        gbc_densityLabel.gridy = 1;
        panel_3.add(densityLabel, gbc_densityLabel);
        
        JPanel panel_4 = new JPanel();
        GridBagConstraints gbc_panel_4 = new GridBagConstraints();
        gbc_panel_4.fill = GridBagConstraints.BOTH;
        gbc_panel_4.gridx = 0;
        gbc_panel_4.gridy = 4;
        panel.add(panel_4, gbc_panel_4);
        GridBagLayout gbl_panel_4 = new GridBagLayout();
        panel_4.setLayout(gbl_panel_4);

        JButton cancelButton = new JButton(Strings.CANCEL.toString());
        cancelButton.addActionListener((ActionEvent e) -> {
            setVisible(false);
            dispose();
        });

        Component horizontalStrut_2 = Box.createVerticalStrut(20);
        GridBagConstraints gbc_horizontalStrut_2 = new GridBagConstraints();
        gbc_horizontalStrut_2.gridy = 0;
        panel_4.add(horizontalStrut_2, gbc_horizontalStrut_2);
        
        GridBagConstraints gbc_cancelButton = new GridBagConstraints();
        gbc_cancelButton.anchor = GridBagConstraints.CENTER;
        gbc_cancelButton.gridy = 1;
        gbc_cancelButton.gridx = 1;
        panel_4.add(cancelButton, gbc_cancelButton);

        JButton addButton = new JButton(Strings.ADD.toString());
        addButton.addActionListener((ActionEvent e) -> {
            ImageIcon imageicon = new ImageIcon(MaterialDetailForm.class.getResource("/sa/shpi/assets/shpismall.jpg"));
            int response = JOptionPane.showConfirmDialog(null, Strings.CONFIRM_DATA, Strings.CONFIRM.toString(), JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE, imageicon);
            if (response == JOptionPane.CANCEL_OPTION) {
                setVisible(true);
            } else if (response == JOptionPane.OK_OPTION) {
                if (!(material.getName() == Strings.NONE.toString())) {
                    StructureUsedMaterials uMaterial = new StructureUsedMaterials(material);
                    parent.addStructureItem(((SectionCategory) sectionComboBox
                            .getSelectedItem()), uMaterial);
                    setVisible(false);
                    dispose();
                } else {
                    JOptionPane.showMessageDialog(MaterialDetailForm.this,
                            Strings.PLEASE_CHOOSE.toString() + " "
                                    + Strings.MATERIAL.toString());
                }
            }
        });
        //Sets the default highlighted button:
        getRootPane().setDefaultButton(addButton);

        GridBagConstraints gbc_addButton = new GridBagConstraints();
        gbc_addButton.anchor = GridBagConstraints.LINE_END;
        gbc_addButton.gridx = 2;
        gbc_addButton.gridy = 1;
        panel_4.add(addButton, gbc_addButton);

        setVisible(true);
    }


    @Override
    public void addEmissionsItem(EmissionsItem eItem) {
        //Add chosen material that was selected:
        this.material = eItem;
        materialPane.setText(eItem.getName());
        emissionsFactorLabel.setText(String.valueOf(material
                .getEmissionsFactor()));
        densityLabel.setText(String.format("%.2f", material.getMass()) + " kg/m^2");
    }
}
