/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.model.structure;

import static java.lang.Math.sqrt;
import sa.shpi.advanced.model.EmissionsItem;
import sa.shpi.advanced.model.Industrial;

/**
 * Dynamic material library with presets to add to structural components. Also
 * contains components that can be used as default structure items.
 * 
 * Specific to External Walls.
 *
 * @author Kyle
 *
 */
public class ExtWallsMaterialsLibrary extends MaterialsLibrary {

    public ExtWallsMaterialsLibrary() {

        /**
         * Emissions items are written in the format: ("item name", emissions
         * factor, mass per unit area)
         */
        lib.clear();
        lib.add(new EmissionsItem("Acero", 0.0117, ((((0.00159 * (sqrt(Industrial.getAreaTotal()) -
            0.00159)) * 6.71 * 4) * 7850) / Industrial.getAreaTotal())));
        lib.add(new EmissionsItem("Concreto", 0.0, 0.0));
        lib.add(new EmissionsItem("Poliestireno - R10", 1.64, ((((0.0508 * (sqrt(Industrial.getAreaTotal()) -
            0.0508)) * 6.71 * 4) * 16) / Industrial.getAreaTotal())));
        lib.add(new EmissionsItem("Poliestireno - R3", 1.64, ((((0.0508 * (sqrt(Industrial.getAreaTotal()) -
            0.0508)) * 6.71 * 4) * 32) / Industrial.getAreaTotal())));
        lib.add(new EmissionsItem("Poliestireno - R1", 1.64, ((((0.0508 * (sqrt(Industrial.getAreaTotal()) -
            0.0508)) * 6.71 * 4) * 48) / Industrial.getAreaTotal())));
    }

    @Override
    public String getUnits() {
        return "kg/m^2";
    }

}
