/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.visual;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import java.awt.GridBagLayout;
import javax.swing.JLabel;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Font;
import java.awt.Dialog.ModalityType;
import java.awt.Window.Type;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Component;
import javax.swing.Box;

import sa.shpi.advanced.model.Strings;

/**
 * Simple dialog box with details about the project.
 *
 * @author Kyle
 *
 */
public class AboutForm extends JDialog {

    private final JPanel contentPanel = new JPanel();

    /**
     * Create the dialog.
     */
    public AboutForm() {
        setType(Type.POPUP);
        setTitle(Strings.ABOUT.toString());
        setResizable(false);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setModalityType(ModalityType.APPLICATION_MODAL);
        setBounds(100, 100, 450, 325);
        getContentPane().setLayout(new BorderLayout());
        contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
        getContentPane().add(contentPanel, BorderLayout.CENTER);
        GridBagLayout gbl_contentPanel = new GridBagLayout();
        gbl_contentPanel.columnWidths = new int[]{0, 0};
        gbl_contentPanel.rowHeights = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        gbl_contentPanel.columnWeights = new double[]{1.0, Double.MIN_VALUE};
        gbl_contentPanel.rowWeights = new double[]{0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE};
        contentPanel.setLayout(gbl_contentPanel);
        {
            Component verticalStrut = Box.createVerticalStrut(20);
            GridBagConstraints gbc_verticalStrut = new GridBagConstraints();
            gbc_verticalStrut.insets = new Insets(0, 0, 5, 0);
            gbc_verticalStrut.gridx = 0;
            gbc_verticalStrut.gridy = 0;
            contentPanel.add(verticalStrut, gbc_verticalStrut);
        }
        {
            JLabel lblTheIndustrialFire = new JLabel(
                    Strings.INDUSTRIAL_EMISSIONS_CALCULATOR.toString() + " v1");
            lblTheIndustrialFire.setFont(new Font("Tahoma", Font.BOLD, 11));
            GridBagConstraints gbc_lblTheResidentialFire = new GridBagConstraints();
            gbc_lblTheResidentialFire.insets = new Insets(0, 0, 5, 0);
            gbc_lblTheResidentialFire.gridx = 0;
            gbc_lblTheResidentialFire.gridy = 1;
            contentPanel.add(lblTheIndustrialFire, gbc_lblTheResidentialFire);
        }
        {
            Component verticalStrut = Box.createVerticalStrut(20);
            GridBagConstraints gbc_verticalStrut = new GridBagConstraints();
            gbc_verticalStrut.insets = new Insets(0, 0, 5, 0);
            gbc_verticalStrut.gridx = 0;
            gbc_verticalStrut.gridy = 2;
            contentPanel.add(verticalStrut, gbc_verticalStrut);
        }
        {
            JLabel lblPartOfThe = new JLabel(
                    "Parte de un Interactive Qualifying Project patrocinado por:");
            lblPartOfThe.setFont(new Font("Tahoma", Font.BOLD, 11));
            GridBagConstraints gbc_lblPartOfThe = new GridBagConstraints();
            gbc_lblPartOfThe.insets = new Insets(0, 0, 5, 0);
            gbc_lblPartOfThe.gridx = 0;
            gbc_lblPartOfThe.gridy = 3;
            contentPanel.add(lblPartOfThe, gbc_lblPartOfThe);
        }
        {
            JLabel lblWorcesterPolytechnicInstitute = new JLabel(
                    "Worcester Polytechnic Institute");
            GridBagConstraints gbc_lblWorcesterPolytechnicInstitute = new GridBagConstraints();
            gbc_lblWorcesterPolytechnicInstitute.insets = new Insets(0, 0, 5, 0);
            gbc_lblWorcesterPolytechnicInstitute.gridx = 0;
            gbc_lblWorcesterPolytechnicInstitute.gridy = 4;
            contentPanel.add(lblWorcesterPolytechnicInstitute,
                    gbc_lblWorcesterPolytechnicInstitute);
        }
        {
            JLabel lblShpiIngenieriaSa = new JLabel("SHPI Ingenieria S.A.");
            GridBagConstraints gbc_lblShpiIngenieriaSa = new GridBagConstraints();
            gbc_lblShpiIngenieriaSa.insets = new Insets(0, 0, 5, 0);
            gbc_lblShpiIngenieriaSa.gridx = 0;
            gbc_lblShpiIngenieriaSa.gridy = 5;
            contentPanel.add(lblShpiIngenieriaSa, gbc_lblShpiIngenieriaSa);
        }
        {
            Component verticalStrut = Box.createVerticalStrut(20);
            GridBagConstraints gbc_verticalStrut = new GridBagConstraints();
            gbc_verticalStrut.insets = new Insets(0, 0, 5, 0);
            gbc_verticalStrut.gridx = 0;
            gbc_verticalStrut.gridy = 6;
            contentPanel.add(verticalStrut, gbc_verticalStrut);
        }
        {
            JLabel lblBy = new JLabel("Creado de:");
            lblBy.setFont(new Font("Tahoma", Font.BOLD, 11));
            GridBagConstraints gbc_lblBy = new GridBagConstraints();
            gbc_lblBy.insets = new Insets(0, 0, 5, 0);
            gbc_lblBy.gridx = 0;
            gbc_lblBy.gridy = 7;
            contentPanel.add(lblBy, gbc_lblBy);
        }
        {
            JLabel lblNames = new JLabel(
                    "Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra");
            GridBagConstraints gbc_lblNames = new GridBagConstraints();
            gbc_lblNames.gridx = 0;
            gbc_lblNames.gridy = 8;
            contentPanel.add(lblNames, gbc_lblNames);
        }
        {
            Component verticalStrut = Box.createVerticalStrut(20);
            GridBagConstraints gbc_verticalStrut = new GridBagConstraints();
            gbc_verticalStrut.insets = new Insets(0, 0, 5, 0);
            gbc_verticalStrut.gridx = 0;
            gbc_verticalStrut.gridy = 9;
            contentPanel.add(verticalStrut, gbc_verticalStrut);
        }
        {
            JLabel lblPrevAuthors = new JLabel("Basado en un trabajo anterior:");
            lblPrevAuthors.setFont(new Font("Tahoma", Font.BOLD, 11));
            GridBagConstraints gbc_lblPrevAuthors = new GridBagConstraints();
            gbc_lblPrevAuthors.insets = new Insets(0, 0, 5, 0);
            gbc_lblPrevAuthors.gridx = 0;
            gbc_lblPrevAuthors.gridy = 10;
            contentPanel.add(lblPrevAuthors, gbc_lblPrevAuthors);

        }
        {
            JLabel lblPrevNames = new JLabel("Alexandra Hardin, Eric Schattshneider, Tyler Stone, Kristina Walker");
            GridBagConstraints gbc_lblPrevNames = new GridBagConstraints();
            gbc_lblPrevNames.gridx = 0;
            gbc_lblPrevNames.gridy = 11;
            contentPanel.add(lblPrevNames, gbc_lblPrevNames);
        }
        {
            JPanel buttonPane = new JPanel();
            buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
            getContentPane().add(buttonPane, BorderLayout.SOUTH);
            {
                JButton okButton = new JButton("OK");
                okButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        setVisible(false);
                        dispose();
                    }
                });
                okButton.setActionCommand("OK");
                buttonPane.add(okButton);
                getRootPane().setDefaultButton(okButton);
            }
        }

        setVisible(true);
    }

}
