/**
* This application has been created as a part of an Interactive Qualifying
* Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
*
* Developer: Kyle Fortin
* Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
*
*/

package emissionsmodelingtool.visual;

import emissionsmodelingtool.model.BuildingType;
import java.awt.Component;
import javax.swing.JFrame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Box;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.BorderFactory;
import javax.swing.JOptionPane;

/**
 *
 * @author kylefortin
 */

public class ModelingToolStartPage {
    
    private JFrame frame;
    private JPanel panel1 = new JPanel();
    private JLabel welcome = new JLabel("¡Bienvenido a la Calculadora de Emisiones!");
    private Component box = Box.createVerticalStrut(20);
    private ImageIcon logoicon = new ImageIcon(this.getClass().getResource("/emissionsmodelingtool/assets/shpismall.jpg"));
    private JLabel logo = new JLabel(logoicon);
    private Component box1 = Box.createVerticalStrut(20);
    private JLabel inst = new JLabel("Por favor, seleccione el tipo de edificio para modelar:");
    private JComboBox type = new JComboBox();
    private Component box2 = Box.createVerticalStrut(20);
    private JLabel inst2 = new JLabel("Pulse '¡Comenzar!' para iniciar la aplicación...");
    private JButton go = new JButton();
    
    private Thread thread;
    private Process proc;
    private ImageIcon flame = new ImageIcon(this.getClass().getResource("/emissionsmodelingtool/assets/flame.gif"));
    private JLabel image = new JLabel(flame);
    private JProgressBar prog = new JProgressBar(0, 100);
    private JLabel loadingpercent = new JLabel();
    private JLabel loadingtype = new JLabel();

    private String filePath = "C:/Program Files/SHPI Calculadora de Emisiones - Windows/";
    
    public static int randInt(int min, int max) {
        Random rand = new Random();
        int randomNum = rand.nextInt((max - min) + 1) + min;
        return randomNum;
    }
    
    public void ModelingTool() {
        initialize();
    }
    
    private void initialize() {
        frame = new JFrame();
        frame.setResizable(false);
        frame.setTitle("SHPI Calculadora de Emisiones");
        frame.setBounds(100, 100, 400, 300);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        GridBagLayout gridBagLayout = new GridBagLayout();
        frame.getContentPane().setLayout(gridBagLayout);
        frame.setVisible(true);
        {
            JPanel panel = new JPanel();
            GridBagConstraints gbc_panel = new GridBagConstraints();
            gbc_panel.fill = GridBagConstraints.BOTH;
            gbc_panel.gridx = 1;
            gbc_panel.gridy = 0;
            frame.getContentPane().add(panel, gbc_panel);
            GridBagLayout gbl_panel = new GridBagLayout();
            panel.setLayout(gbl_panel);
            {
                //Dummy thread to display progress bar activity.
                thread = new Thread(){
                    @Override
                    public void run(){
                        for(int i = prog.getMinimum() ; i < prog.getMaximum() + 1; i++){
                            final int percent = i;
                            SwingUtilities.invokeLater(() -> {
                                prog.setValue(percent);
                                loadingpercent.setText(String.valueOf(percent) + "%");
                            });
                            try {
                                Thread.sleep(randInt(0, 20));
                            } catch (InterruptedException e) {}
                        }
                        frame.setVisible(false);
                        frame.dispose();
                    }
                };
            }
            {
                //Welcome message
                GridBagConstraints gbc_welcome = new GridBagConstraints();
                gbc_welcome.anchor = GridBagConstraints.CENTER;
                gbc_welcome.gridy = 0;
                panel.add(welcome, gbc_welcome);
            }
            {
                GridBagConstraints gbc_box1 = new GridBagConstraints();
                gbc_box1.anchor = GridBagConstraints.CENTER;
                gbc_box1.gridy = 1;
                panel.add(box1, gbc_box1);
            }
            {
                GridBagConstraints gbc_panel1 = new GridBagConstraints();
                gbc_panel1.fill = GridBagConstraints.BOTH;
                gbc_panel1.gridy = 2;
                panel.add(panel1, gbc_panel1);
                GridBagLayout gbl_panel1 = new GridBagLayout();
                panel1.setLayout(gbl_panel1);
                {
                    //The SHPI logo.
                    logo.setBorder(BorderFactory.createRaisedBevelBorder());
                    GridBagConstraints gbc_logo = new GridBagConstraints();
                    gbc_logo.anchor = GridBagConstraints.CENTER;
                    panel1.add(logo, gbc_logo);
                }
            }
            {
                GridBagConstraints gbc_box2 = new GridBagConstraints();
                gbc_box2.anchor = GridBagConstraints.CENTER;
                gbc_box2.gridy = 3;
                panel.add(box2, gbc_box2);
            }
            {
                //Instructions to choose type of building.
                GridBagConstraints gbc_inst = new GridBagConstraints();
                gbc_inst.anchor = GridBagConstraints.CENTER;
                gbc_inst.gridy = 4;
                panel.add(inst, gbc_inst);
            }
            {
                //Combo box to select building type.
                type.setSelectedItem(BuildingType.WARE);
                type.setModel(new DefaultComboBoxModel(BuildingType.values()));
                GridBagConstraints gbc_type = new GridBagConstraints();
                gbc_type.anchor = GridBagConstraints.CENTER;
                gbc_type.gridy = 5;
                panel.add(type, gbc_type);
            }
            {
                GridBagConstraints gbc_box = new GridBagConstraints();
                gbc_box.anchor = GridBagConstraints.CENTER;
                gbc_box.gridy = 6;
                panel.add(box, gbc_box);
            }
            {
                //Instructions to click on go button.
                GridBagConstraints gbc_inst2 = new GridBagConstraints();
                gbc_inst2.anchor = GridBagConstraints.CENTER;
                gbc_inst2.gridy = 7;
                panel.add(inst2, gbc_inst2);
            }
            {
                //Button to launch selected application.
                go.addActionListener((ActionEvent e) -> {
                    File file;
                    //Retrieve .jar file for selected model.
                    if (type.getSelectedItem().equals(BuildingType.HOSPITAL)) {
                        file = new File(filePath + "/Hospital/Hospital.jar");
                        //If files exists, build process to run .jar model.
                        if (file.exists() && file.isFile()){
                            ProcessBuilder pb = new ProcessBuilder("java", "-jar", filePath + "/Hospital/Hospital.jar");
                            try {
                                proc = pb.start();
                            } catch (IOException ex) {
                                Logger.getLogger(ModelingToolStartPage.class.getName()).log(Level.SEVERE, null, ex);
                            }
                        }
                        //If file !exists, display error and dispose.
                        else {
                            JOptionPane.showMessageDialog(null, "Lo siento.", null, JOptionPane.INFORMATION_MESSAGE, logoicon);
                            frame.dispose();
                            frame.setVisible(false);
                        }
                    } else if (type.getSelectedItem().equals(BuildingType.IND)) {
                        file = new File(filePath + "/Industrial/Industrial.jar");
                        if (file.exists() && file.isFile()){
                            ProcessBuilder pb = new ProcessBuilder("java", "-jar", filePath + "Industrial/Industrial.jar");
                            try {
                                proc = pb.start();
                            } catch (IOException ex) {
                                Logger.getLogger(ModelingToolStartPage.class.getName()).log(Level.SEVERE, null, ex);
                            }
                        } else {
                            JOptionPane.showMessageDialog(null, "Lo siento.", null, JOptionPane.INFORMATION_MESSAGE, logoicon);
                            frame.dispose();
                            frame.setVisible(false);
                        }
                    } else if (type.getSelectedItem().equals(BuildingType.WARE)) {
                        file = new File(filePath + "/Bodega/Bodega.jar");
                        if (file.exists() && file.isFile()){
                            ProcessBuilder pb = new ProcessBuilder("java", "-jar", filePath + "Bodega/Bodega.jar");
                            try {
                                proc = pb.start();
                            } catch (IOException ex) {
                                Logger.getLogger(ModelingToolStartPage.class.getName()).log(Level.SEVERE, null, ex);
                            }
                        } else {
                            JOptionPane.showMessageDialog(null, "Lo siento.", null, JOptionPane.INFORMATION_MESSAGE, logoicon);
                            frame.dispose();
                            frame.setVisible(false);
                        }
                    } else if (type.getSelectedItem().equals(BuildingType.RESIDENTIAL)) {
                        file = new File(filePath + "/Residencial/Residencial.jar");
                        if (file.exists() && file.isFile()){
                            ProcessBuilder pb = new ProcessBuilder("java", "-jar", filePath + "Residencial/Residencial.jar");
                            try {
                                proc = pb.start();
                            } catch (IOException ex) {
                                Logger.getLogger(ModelingToolStartPage.class.getName()).log(Level.SEVERE, null, ex);
                            }
                        } else {
                            JOptionPane.showMessageDialog(null, "Lo siento.", null, JOptionPane.INFORMATION_MESSAGE, logoicon);
                            frame.dispose();
                            frame.setVisible(false);
                        }
                    }
                    //If .jar is laucnhed, display loading screen.
                    panel1.setVisible(false);
                    logo.setVisible(false);
                    welcome.setVisible(false);
                    inst.setVisible(false);
                    type.setVisible(false);
                    inst2.setVisible(false);
                    go.setVisible(false);
                    prog.setVisible(true);
                    image.setVisible(true);
                    loadingtype.setText("Ahora cargar " + type.getSelectedItem().toString() + "...");
                    loadingtype.setVisible(true);
                    loadingpercent.setVisible(true);
                    thread.start();
                });
                go.setText("Comenzar");
                GridBagConstraints gbc_go = new GridBagConstraints();
                gbc_go.anchor = GridBagConstraints.CENTER;
                gbc_go.gridy = 8;
                panel.add(go, gbc_go);
            }
            {
                //Flame .gif
                GridBagConstraints gbc_image = new GridBagConstraints();
                gbc_image.anchor = GridBagConstraints.CENTER;
                gbc_image.gridy = 1;
                gbc_image.gridx = 1;
                panel.add(image, gbc_image);
                image.setVisible(false);
            }
            {
                //Display dummy loading percent.
                GridBagConstraints gbc_loadingpercent = new GridBagConstraints();
                gbc_loadingpercent.anchor = GridBagConstraints.CENTER;
                gbc_loadingpercent.gridy = 2;
                gbc_loadingpercent.gridx = 1;
                panel.add(loadingpercent, gbc_loadingpercent);
                loadingpercent.setVisible(false);
                
            }
            {
                //Display dummy progress bar.
                GridBagConstraints gbc_prog = new GridBagConstraints();
                gbc_prog.anchor = GridBagConstraints.CENTER;
                gbc_prog.gridy = 2;
                gbc_prog.gridx = 1;
                panel.add(prog, gbc_prog);
                prog.setVisible(false);
            }
            {
                //Display loading message with selected application.
                GridBagConstraints gbc_loadingtype = new GridBagConstraints();
                gbc_loadingtype.anchor = GridBagConstraints.CENTER;
                gbc_loadingtype.gridy = 3;
                gbc_loadingtype.gridx = 1;
                panel.add(loadingtype, gbc_loadingtype);
                loadingtype.setVisible(false);
            }
        }
    }
}


